@@ -20,7 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <pthread.h>
+#include <glib.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -48,27 +49,47 @@
 #define COTHREAD_MAXTHREADS 16
 #define COTHREAD_STACKSIZE (STACK_SIZE/COTHREAD_MAXTHREADS)
 
-static void 	cothread_destroy 	(cothread_state *thread);
+static void 	cothread_destroy 	(cothread_state *cothread);
 
 struct _cothread_context
 {
   cothread_state *cothreads[COTHREAD_MAXTHREADS]; /* array of cothread states */
   int ncothreads;
   int current;
+  unsigned long stack_top;
   GHashTable *data;
+  GThread *thread;
 };
 
-
-/* this _cothread_ctx_key is used as a pthread key to the thread's context
- * a pthread key is a "pointer" to memory space that is/can be different
- * (ie. private) for each thread.  The key itself is shared among threads,
- * so it only needs to be initialized once.
- */
-static pthread_key_t _cothread_ctx_key = -1;
-
 /* Disabling this define allows you to shut off a few checks in
  * cothread_switch.  This likely will speed things up fractionally */
-/* #define COTHREAD_PARANOID */
+#define COTHREAD_PARANOID
+
+
+/* this _cothread_ctx_key is used as a GThread key to the thread's context
+ * a GThread key is a "pointer" to memory space that is/can be different
+ * (ie. private) for each thread.  The key itself is shared among threads,
+ * so it only needs to be initialized once.
+ */
+static GStaticPrivate _cothread_ctx_key = G_STATIC_PRIVATE_INIT;
+
+/*
+ * This should only after context init, since we do checking.
+ */
+static cothread_context *
+cothread_get_current_context (void)
+{
+  cothread_context *ctx;
+
+  ctx = g_static_private_get (&_cothread_ctx_key);
+  g_assert(ctx);
+
+#ifdef COTHREAD_PARANOID
+  g_assert (ctx->thread == g_thread_self());
+#endif
+
+  return ctx;
+}
 
 /**
  * cothread_context_init:
@@ -80,33 +101,52 @@ static pthread_key_t _cothread_ctx_key = -1;
 cothread_context *
 cothread_context_init (void)
 {
+  cothread_context *ctx;
+  void *sp;
+
+  /* if there already is a cotread context for this thread,
+   * just return it */
+  ctx = g_static_private_get (&_cothread_ctx_key);
+  if (ctx) {
+    GST_INFO (GST_CAT_COTHREADS, 
+	      "returning private _cothread_ctx_key %p", ctx);
+    return ctx;
+  }
+
   /*
    * initalize the whole of the cothreads context 
    */
-  cothread_context *ctx = (cothread_context *) g_malloc (sizeof (cothread_context));
+  ctx = (cothread_context *) g_malloc (sizeof (cothread_context));
 
   /* we consider the initiating process to be cothread 0 */
   ctx->ncothreads = 1;
   ctx->current = 0;
   ctx->data = g_hash_table_new (g_str_hash, g_str_equal);
+  ctx->thread = g_thread_self();
 
   GST_INFO (GST_CAT_COTHREADS, "initializing cothreads");
 
-  /* initialize the cothread key (for pthread space) if not done yet */
-  if (_cothread_ctx_key == (pthread_key_t) -1) {
-    if (pthread_key_create (&_cothread_ctx_key, NULL) != 0) {
-      perror ("pthread_key_create");
-      return NULL;
-    }
-  }
-
   /* set this thread's context pointer */
-  pthread_setspecific (_cothread_ctx_key, ctx);
+  GST_INFO (GST_CAT_COTHREADS, "setting private _cothread_ctx_key to %p in thread %p",
+	    ctx,g_thread_self());
+  g_static_private_set (&_cothread_ctx_key, ctx, NULL);
+
+  g_assert(ctx == cothread_get_current_context());
 
   /* clear the cothread data */
-
   memset (ctx->cothreads, 0, sizeof (ctx->cothreads));
 
+  sp = CURRENT_STACK_FRAME;
+  /* FIXME this may not be 64bit clean
+   *       could use casts to uintptr_t from inttypes.h
+   *       if only all platforms had inttypes.h
+   */
+  /* stack_top is the address of the first byte past our stack segment. */
+  /* FIXME: an assumption is made that the stack segment is STACK_SIZE
+   * aligned. */
+  ctx->stack_top = ((gulong) sp | (STACK_SIZE - 1)) + 1;
+  GST_DEBUG (GST_CAT_COTHREADS, "stack top is 0x%08lx", ctx->stack_top);
+
   /*
    * initialize the 0th cothread
    */
@@ -119,7 +159,6 @@ cothread_context_init (void)
   ctx->cothreads[0]->priv = NULL;
   ctx->cothreads[0]->flags = COTHREAD_STARTED;
   ctx->cothreads[0]->sp = (void *) CURRENT_STACK_FRAME;
-  ctx->cothreads[0]->pc = 0;
 
   GST_INFO (GST_CAT_COTHREADS, "0th cothread is %p at sp:%p", 
             ctx->cothreads[0], ctx->cothreads[0]->sp);
@@ -139,15 +178,25 @@ cothread_context_free (cothread_context *ctx)
   gint i;
 
   g_return_if_fail (ctx != NULL);
+  g_assert (ctx->thread == g_thread_self());
+  g_assert (ctx->current == 0);
 
   GST_INFO (GST_CAT_COTHREADS, "free cothread context");
 
-  for (i = 0; i < COTHREAD_MAXTHREADS; i++) {
+  for (i = 1; i < COTHREAD_MAXTHREADS; i++) {
     if (ctx->cothreads[i]) {
       cothread_destroy (ctx->cothreads[i]);
     }
   }
+  if (ctx->cothreads[0]) {
+    g_free(ctx->cothreads[0]);
+    ctx->cothreads[0] = NULL;
+  }
   g_hash_table_destroy (ctx->data);
+  /* make sure we free the private key for cothread context */
+  GST_INFO (GST_CAT_COTHREADS, "setting private _cothread_ctx_key to NULL in thread %p",
+	    g_thread_self());
+  g_static_private_set (&_cothread_ctx_key, NULL, NULL);
   g_free (ctx);
 }
 
@@ -163,16 +212,16 @@ cothread_state*
 cothread_create (cothread_context *ctx)
 {
   cothread_state *cothread;
-  void *sp;
   void *mmaped = 0;
-  guchar *stack_end;
   gint slot = 0;
+  unsigned long page_size;
 
   g_return_val_if_fail (ctx != NULL, NULL);
+  g_assert (ctx->thread == g_thread_self());
 
   if (ctx->ncothreads == COTHREAD_MAXTHREADS) {
     /* this is pretty fatal */
-    g_warning ("cothread_create: attempt to create > COTHREAD_MAXTHREADS\n");
+    g_warning ("cothread_create: attempt to create > COTHREAD_MAXTHREADS");
     return NULL;
   }
   /* find a free spot in the stack, note slot 0 has the main thread */
@@ -188,34 +237,50 @@ cothread_create (cothread_context *ctx)
 
   GST_DEBUG (GST_CAT_COTHREADS, "Found free cothread slot %d", slot);
 
-  sp = CURRENT_STACK_FRAME;
-  /* FIXME this may not be 64bit clean
-   *       could use casts to uintptr_t from inttypes.h
-   *       if only all platforms had inttypes.h
-   */
-  /* FIXME: a little explanation on what this REALLY means would be nice ;) */
-  stack_end = (guchar *) ((gulong) sp & ~(STACK_SIZE - 1));
-
   /* cothread stack space of the thread is mapped in reverse, with cothread 0
    * stack space at the top */
-  cothread = (cothread_state *) (stack_end + ((slot - 1) * COTHREAD_STACKSIZE));
-  GST_DEBUG (GST_CAT_COTHREADS, 
-             "mmap   cothread slot stack from %p to %p (size 0x%lx)", 
-	     cothread, cothread + COTHREAD_STACKSIZE, 
-	     (long) COTHREAD_STACKSIZE);
+  cothread = (cothread_state *) (ctx->stack_top - (slot + 1) * COTHREAD_STACKSIZE);
+  GST_DEBUG (GST_CAT_COTHREADS, "cothread pointer is %p", cothread);
 
-  GST_DEBUG (GST_CAT_COTHREADS, "going into mmap");
-  /* the mmap is used to reserve part of the stack
-   * ie. we state explicitly that we are going to use it */
-  mmaped = mmap ((void *) cothread, COTHREAD_STACKSIZE,
-	          PROT_READ | PROT_WRITE | PROT_EXEC, 
-		  MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+#if 0
+  /* This tests to see whether or not we can grow down the stack */
+  {
+    unsigned long ptr;
+    for(ptr=ctx->stack_top - 4096; ptr > (unsigned long)cothread; ptr -= 4096){
+      GST_DEBUG (GST_CAT_COTHREADS, "touching location 0x%08lx", ptr);
+      *(volatile unsigned int *)ptr = *(volatile unsigned int *)ptr;
+      GST_DEBUG (GST_CAT_COTHREADS, "ok (0x%08x)", *(unsigned int *)ptr);
+    }
+  }
+#endif
+
+#ifdef _SC_PAGESIZE
+  page_size = sysconf(_SC_PAGESIZE);
+#else
+  page_size = getpagesize();
+#endif
+
+  /* The mmap is necessary on Linux/i386, and possibly others, since the
+   * kernel is picky about when we can expand our stack. */
+  GST_DEBUG (GST_CAT_COTHREADS, "mmaping %p, size 0x%08x", cothread,
+             COTHREAD_STACKSIZE);
+  /* Remap with a guard page. This decreases our stack size by 8 kB (for
+   * 4 kB pages) and also wastes almost 4 kB for the cothreads
+   * structure */
+  munmap((void *)cothread, COTHREAD_STACKSIZE);
+  mmaped = mmap ((void *) cothread, page_size,
+                 PROT_READ | PROT_WRITE,
+                 MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+  mmaped = mmap (((void *) cothread) + page_size * 2,
+		 COTHREAD_STACKSIZE - page_size * 2,
+                 PROT_READ | PROT_WRITE | PROT_EXEC,
+                 MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
   GST_DEBUG (GST_CAT_COTHREADS, "coming out of mmap");
   if (mmaped == MAP_FAILED) {
     perror ("mmap'ing cothread stack space");
     return NULL;
   }
-  if (mmaped != cothread) {
+  if (mmaped != (void *)cothread + page_size * 2) {
     g_warning ("could not mmap requested memory for cothread");
     return NULL;
   }
@@ -228,8 +293,8 @@ cothread_create (cothread_context *ctx)
   cothread->flags = 0;
   cothread->priv = NULL;
   cothread->sp = ((guchar *) cothread + COTHREAD_STACKSIZE);
-  cothread->top_sp = cothread->sp; /* for debugging purposes 
-				      to detect stack overruns */
+  cothread->stack_size = COTHREAD_STACKSIZE - page_size * 2;
+  cothread->stack_base = (void *)cothread + 2 * page_size;
 
   GST_INFO (GST_CAT_COTHREADS, 
             "created cothread #%d in slot %d: %p at sp:%p", 
@@ -256,7 +321,10 @@ cothread_free (cothread_state *cothread)
             cothread->cothreadnum);
 
   /* we simply flag the cothread for destruction here */
-  cothread->flags |= COTHREAD_DESTROYED;
+  if (cothread)
+    cothread->flags |= COTHREAD_DESTROYED;
+  else
+    g_warning ("somebody set up us the bomb");
 }
 
 static void
@@ -269,73 +337,27 @@ cothread_destroy (cothread_state *cothread)
 
   cothreadnum = cothread->cothreadnum;
   ctx = cothread->ctx;
+  g_assert (ctx->thread == g_thread_self());
+  g_assert (ctx == cothread_get_current_context());
 
   GST_INFO (GST_CAT_COTHREADS, "destroy cothread %d %p %d", 
             cothreadnum, cothread, ctx->current);
 
+  /* cothread 0 needs to be destroyed specially */
+  g_assert(cothreadnum != 0);
+
   /* we have to unlock here because we might be switched out 
    * with the lock held */
   cothread_unlock (cothread);
 
-  if (cothreadnum == 0) 
-  {
-    GST_INFO (GST_CAT_COTHREADS,
-	      "trying to destroy cothread 0 with %d cothreads left", 
-	     ctx->ncothreads);
-    if (ctx->ncothreads > 1)
-    {
-      /* we're trying to destroy cothread 0 when there are still cothreads
-       * active, so kill those first */
-      int i;
+  /* doing cleanups of the cothread create */
+  GST_DEBUG (GST_CAT_COTHREADS, "destroy cothread %d with magic number 0x%x",
+           cothreadnum, cothread->magic_number);
+  g_assert (cothread->magic_number == COTHREAD_MAGIC_NUMBER);
 
-      for (i = 1; i < COTHREAD_MAXTHREADS; ++i)
-      {
-	if (ctx->cothreads[i] != NULL)
-	{
-	  cothread_destroy (ctx->cothreads[i]);
-	  GST_INFO (GST_CAT_COTHREADS,
-	            "destroyed cothread %d, %d cothreads left\n", 
-		    i, ctx->ncothreads);
-	}
-      }
-    }
-    g_assert (ctx->ncothreads == 1);
-    g_free (cothread);
-  }
-  else {
-  /*  int res; 
-   *  Replaced with version below until cothreads issues solved */
-   int res = 0;
-    
-    /* doing cleanups of the cothread create */
-    GST_DEBUG (GST_CAT_COTHREADS, "destroy cothread %d with magic number 0x%x",
-             cothreadnum, cothread->magic_number);
-    g_assert (cothread->magic_number == COTHREAD_MAGIC_NUMBER);
+  g_assert (cothread->priv == NULL);
 
-    g_assert (cothread->priv == NULL);
-
-    GST_DEBUG (GST_CAT_COTHREADS, 
-               "munmap cothread slot stack from %p to %p (size 0x%lx)", 
-  	       cothread, cothread + COTHREAD_STACKSIZE, 
-	       (long) COTHREAD_STACKSIZE);
-/*    res = munmap (thread, COTHREAD_STACKSIZE);
- *    Commented out waiting for resolution for cothread issue */
-    if (res != 0)
-    {
-      switch (res)
-      {
-	case EINVAL:
-	  g_warning ("munmap doesn't like start %p or length %d\n",
-	             cothread, COTHREAD_STACKSIZE);
-	  break;
-	default:
-	  g_warning ("Thomas was too lazy to check for all errors, "
-	             "so I can't tell you what is wrong.\n");
-	  break;
-      }
-    }
-  }
-  GST_DEBUG (GST_CAT_COTHREADS, "munmap done\n");
+  memset(cothread,0,sizeof(*cothread));
 
   ctx->cothreads[cothreadnum] = NULL;
   ctx->ncothreads--;
@@ -343,7 +365,7 @@ cothread_destroy (cothread_state *cothread)
 
 /**
  * cothread_setfunc:
- * @thread: the cothread state
+ * @cothread: the cothread state
  * @func: the function to call
  * @argc: argument count for the cothread function
  * @argv: arguments for the cothread function
@@ -351,26 +373,23 @@ cothread_destroy (cothread_state *cothread)
  * Set the cothread function
  */
 void
-cothread_setfunc (cothread_state * thread, cothread_func func, int argc, char **argv)
+cothread_setfunc (cothread_state* cothread, cothread_func func, int argc, char **argv)
 {
-  thread->func = func;
-  thread->argc = argc;
-  thread->argv = argv;
-  thread->pc = (void *) func;
+  cothread->func = func;
+  cothread->argc = argc;
+  cothread->argv = argv;
 }
 
 /**
  * cothread_stop:
- * @thread: the cothread to stop
+ * @cothread: the cothread to stop
  *
  * Stop the cothread and reset the stack and program counter.
  */
 void
-cothread_stop (cothread_state * thread)
+cothread_stop (cothread_state* cothread)
 {
-  thread->flags &= ~COTHREAD_STARTED;
-  thread->pc = 0;
-  thread->sp = thread->top_sp;
+  cothread->flags &= ~COTHREAD_STARTED;
 }
 
 /**
@@ -381,9 +400,11 @@ cothread_stop (cothread_state * thread)
  *
  * Returns: the #cothread_state of the main (0th) cothread.
  */
-cothread_state *
-cothread_main (cothread_context * ctx)
+cothread_state*
+cothread_main (cothread_context* ctx)
 {
+  g_assert (ctx->thread == g_thread_self());
+
   GST_DEBUG (GST_CAT_COTHREADS, "returning %p, the 0th cothread", 
              ctx->cothreads[0]);
   return ctx->cothreads[0];
@@ -392,14 +413,14 @@ cothread_main (cothread_context * ctx)
 /**
  * cothread_current_main:
  *
- * Get the main thread in the current pthread.
+ * Get the main thread in the current GThread.
  *
- * Returns: the #cothread_state of the main (0th) thread in the current pthread
+ * Returns: the #cothread_state of the main (0th) thread in the current GThread
  */
-cothread_state *
+cothread_state*
 cothread_current_main (void)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
+  cothread_context *ctx = cothread_get_current_context();
 
   return ctx->cothreads[0];
 }
@@ -411,10 +432,10 @@ cothread_current_main (void)
  *
  * Returns: the #cothread_state of the current cothread
  */
-cothread_state *
+cothread_state*
 cothread_current (void)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
+  cothread_context *ctx = cothread_get_current_context();
 
   return ctx->cothreads[ctx->current];
 }
@@ -422,15 +443,26 @@ cothread_current (void)
 static void
 cothread_stub (void)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
-  register cothread_state *thread = ctx->cothreads[ctx->current];
+  cothread_context *ctx = cothread_get_current_context();
+  cothread_state *cothread = ctx->cothreads[ctx->current];
 
   GST_DEBUG_ENTER ("");
 
-  thread->flags |= COTHREAD_STARTED;
+  GST_DEBUG (GST_CAT_COTHREADS, "stack addr %p", &ctx);
+
+  cothread->flags |= COTHREAD_STARTED;
 
   while (TRUE) {
-    thread->func (thread->argc, thread->argv);
+    cothread->func (cothread->argc, cothread->argv);
+
+    GST_DEBUG (GST_CAT_COTHREADS, "cothread[%d] thread->func exited", ctx->current);
+
+    GST_DEBUG (GST_CAT_COTHREADS, "sp=%p", CURRENT_STACK_FRAME);
+    GST_DEBUG (GST_CAT_COTHREADS, "ctx=%p current=%p", ctx,cothread_get_current_context());
+    g_assert (ctx == cothread_get_current_context());
+
+    g_assert (ctx->current != 0);
+
     /* we do this to avoid ever returning, we just switch to 0th thread */
     cothread_switch (cothread_main (ctx));
   }
@@ -444,63 +476,65 @@ cothread_stub (void)
  *
  * Returns: the current cothread id
  */
-int cothread_getcurrent (void) __attribute__ ((no_instrument_function));
+int cothread_getcurrent (void) G_GNUC_NO_INSTRUMENT;
+
 int
 cothread_getcurrent (void)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
+  cothread_context *ctx = cothread_get_current_context();
 
   if (!ctx)
     return -1;
+
   return ctx->current;
 }
 
 /**
  * cothread_set_private:
- * @thread: the cothread state
+ * @cothread: the cothread state
  * @data: the data
  *
  * set private data for the cothread.
  */
 void
-cothread_set_private (cothread_state *thread, gpointer data)
+cothread_set_private (cothread_state *cothread, gpointer data)
 {
-  thread->priv = data;
+  cothread->priv = data;
 }
 
 /**
  * cothread_context_set_data:
- * @thread: the cothread state
+ * @cothread: the cothread state
  * @key: a key for the data
  * @data: the data
  *
  * adds data to a cothread
  */
 void
-cothread_context_set_data (cothread_state *thread, gchar *key, gpointer data)
+cothread_context_set_data (cothread_state *cothread, gchar *key, gpointer data)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
+  cothread_context *ctx = cothread_get_current_context();
 
   g_hash_table_insert (ctx->data, key, data);
 }
 
 /**
  * cothread_get_private:
- * @thread: the cothread state
+ * @cothread: the cothread state
  *
  * get the private data from the cothread
  *
  * Returns: the private data of the cothread
  */
 gpointer
-cothread_get_private (cothread_state *thread)
+cothread_get_private (cothread_state *cothread)
 {
-  return thread->priv;
+  return cothread->priv;
 }
 
 /**
  * cothread_context_get_data:
- * @thread: the cothread state
+ * @cothread: the cothread state
  * @key: a key for the data
  *
  * get data from the cothread
@@ -508,82 +542,35 @@ cothread_get_private (cothread_state *thread)
  * Returns: the data associated with the key
  */
 gpointer
-cothread_context_get_data (cothread_state * thread, gchar * key)
+cothread_context_get_data (cothread_state *cothread, gchar *key)
 {
-  cothread_context *ctx = pthread_getspecific (_cothread_ctx_key);
+  cothread_context *ctx = cothread_get_current_context();
 
   return g_hash_table_lookup (ctx->data, key);
 }
 
-/**
- * cothreads_stackquery:
- * @stack: Will be set to point to the allocated stack location
- * @stacksize: Will be set to the size of the allocated stack
- *
- *  Returns: #TRUE on success, #FALSE otherwise.
- */
-gboolean
-cothread_stackquery (void **stack, glong* stacksize)
-{
-  /* use either
-   * - posix_memalign to allocate a 2M-aligned, 2M stack 
-   * or
-   * - valloc 
-   *
-   * memory allocated by either of these two can be freed using free () 
-   * FIXME: define how the stack grows */
-
-#ifdef HAVE_POSIX_MEMALIGN
-  int retval = posix_memalign (stack, STACK_SIZE, STACK_SIZE);
-  if (retval != 0)
-  {
-    g_warning ("Could not posix_memalign stack !\n");
-    if (retval == EINVAL)
-      g_warning ("The alignment parameter %d was not a power of two !\n",
-	         STACK_SIZE);
-    if (retval == ENOMEM)
-      g_warning ("Insufficient memory to allocate the request of %d !\n",
-	         STACK_SIZE);
-    *stacksize = 0;
-    return FALSE;
-  }
-  GST_DEBUG (GST_CAT_THREAD, "have  posix_memalign at %p of size %d",
-             (void *) *stack, STACK_SIZE);
-#else
-  if ((*stack = valloc (STACK_SIZE)) != 0)
-  {
-    g_warning ("Could not valloc stack !\n");
-    return FALSE;
-  }
-  GST_DEBUG (GST_CAT_THREAD, "have  valloc at %p of size %d",
-             (void *) *stack, STACK_SIZE);
-#endif
-
-  GST_DEBUG (GST_CAT_COTHREADS, 
-             "Got new cothread stack from %p to %p (size %ld)",
-             *stack, *stack + STACK_SIZE - 1, (long) STACK_SIZE);
-  *stacksize = STACK_SIZE;
-  return TRUE;
-}
-
 /**
  * cothread_switch:
- * @thread: cothread state to switch to
+ * @cothread: cothread state to switch to
  *
  * Switches to the given cothread state
  */
 void
-cothread_switch (cothread_state * thread)
+cothread_switch (cothread_state *cothread)
 {
   cothread_context *ctx;
   cothread_state *current;
   int enter;
 
 #ifdef COTHREAD_PARANOID
-  if (thread == NULL)
+  if (cothread == NULL)
     goto nothread;
 #endif
-  ctx = thread->ctx;
+  ctx = cothread->ctx;
+
+  /* paranoia check to make sure we're in the right thread */
+  g_assert (ctx->thread == g_thread_self());
+
 #ifdef COTHREAD_PARANOID
   if (ctx == NULL)
     goto nocontext;
@@ -594,15 +581,17 @@ cothread_switch (cothread_state * thread)
   if (current == NULL)
     goto nocurrent;
 #endif
-  if (current == thread)
+  if (current == cothread)
     goto selfswitch;
 
 
   /* find the number of the thread to switch to */
   GST_INFO (GST_CAT_COTHREAD_SWITCH, 
             "switching from cothread #%d to cothread #%d",
-	    ctx->current, thread->cothreadnum);
-  ctx->current = thread->cothreadnum;
+	    ctx->current, cothread->cothreadnum);
+  ctx->current = cothread->cothreadnum;
+
+  g_static_private_set (&_gst_debug_cothread_index, (void *)ctx->current, NULL);
 
   /* save the current stack pointer, frame pointer, and pc */
 #ifdef GST_ARCH_PRESETJMP
@@ -611,32 +600,45 @@ cothread_switch (cothread_state * thread)
   enter = setjmp (current->jmp);
   if (enter != 0) {
     GST_DEBUG (GST_CAT_COTHREADS, 
-	       "enter cothread #%d %d %p<->%p (%d) %p", 
-	       current->cothreadnum, enter, current->sp, current->top_sp, 
-	       (char*) current->top_sp - (char*) current->sp, current->jmp);
+	       "enter cothread #%d %d sp=%p jmpbuf=%p", 
+	       current->cothreadnum, enter, current->sp, current->jmp);
     return;
   }
-  GST_DEBUG (GST_CAT_COTHREADS, "exit cothread #%d %d %p<->%p (%d) %p", 
-             current->cothreadnum, enter, current->sp, current->top_sp, 
-	     (char *) current->top_sp - (char *) current->sp, current->jmp);
+  GST_DEBUG (GST_CAT_COTHREADS, "exit cothread #%d %d sp=%p jmpbuf=%p", 
+             current->cothreadnum, enter, current->sp, current->jmp);
   enter = 1;
 
   if (current->flags & COTHREAD_DESTROYED) {
     cothread_destroy (current);
   }
 
-  GST_DEBUG (GST_CAT_COTHREADS, "set stack to %p", thread->sp);
+  GST_DEBUG (GST_CAT_COTHREADS, "set stack to %p", cothread->sp);
   /* restore stack pointer and other stuff of new cothread */
-  if (thread->flags & COTHREAD_STARTED) {
-    GST_DEBUG (GST_CAT_COTHREADS, "in thread %p", thread->jmp);
+  if (cothread->flags & COTHREAD_STARTED) {
+    GST_DEBUG (GST_CAT_COTHREADS, "via longjmp() jmpbuf %p", cothread->jmp);
     /* switch to it */
-    longjmp (thread->jmp, 1);
+    longjmp (cothread->jmp, 1);
   }
   else {
-    GST_ARCH_SETUP_STACK ((char*)thread->sp);
-    GST_ARCH_SET_SP (thread->sp);
+#ifdef HAVE_MAKECONTEXT
+    ucontext_t ucp;
+
+    GST_DEBUG (GST_CAT_COTHREADS, "making context");
+
+    g_assert (cothread != cothread_main(ctx));
+
+    getcontext (&ucp);
+    ucp.uc_stack.ss_sp = (void *)cothread->stack_base;
+    ucp.uc_stack.ss_size = cothread->stack_size;
+    makecontext (&ucp, cothread_stub, 0);
+    setcontext (&ucp);
+#else
+    GST_ARCH_SETUP_STACK ((char*)cothread->sp);
+    GST_ARCH_SET_SP (cothread->sp);
     /* start it */
     GST_ARCH_CALL (cothread_stub);
+#endif
+
     GST_DEBUG (GST_CAT_COTHREADS, "exit thread ");
     ctx->current = 0;
   }
@@ -645,52 +647,52 @@ cothread_switch (cothread_state * thread)
 
 #ifdef COTHREAD_PARANOID
 nothread:
-  g_warning ("cothread: can't switch to NULL cothread!\n");
+  g_warning ("cothread: can't switch to NULL cothread!");
   return;
 nocontext:
-  g_warning ("cothread: there's no context, help!\n");
+  g_warning ("cothread: there's no context, help!");
   exit (2);
 nocurrent:
-  g_warning ("cothread: there's no current thread, help!\n");
+  g_warning ("cothread: there's no current thread, help!");
   exit (2);
 #endif /* COTHREAD_PARANOID */
 selfswitch:
-  g_warning ("cothread: trying to switch to same thread, legal but not necessary\n");
+  g_warning ("cothread: trying to switch to same thread, legal but not necessary");
   return;
 }
 
 /**
  * cothread_lock:
- * @thread: cothread state to lock
+ * @cothread: cothread state to lock
  *
  * Locks the cothread state.
  */
 void
-cothread_lock (cothread_state * thread)
+cothread_lock (cothread_state *cothread)
 {
 }
 
 /**
  * cothread_trylock:
- * @thread: cothread state to try to lock
+ * @cothread: cothread state to try to lock
  *
  * Try to lock the cothread state
  *
  * Returns: TRUE if the cothread could be locked.
  */
 gboolean
-cothread_trylock (cothread_state * thread)
+cothread_trylock (cothread_state *cothread)
 {
   return TRUE;
 }
 
 /**
  * cothread_unlock:
- * @thread: cothread state to unlock
+ * @cothread: cothread state to unlock
  *
  * Unlock the cothread state.
  */
 void
-cothread_unlock (cothread_state * thread)
+cothread_unlock (cothread_state *cothread)
 {
 }
