@@ -49,8 +49,6 @@ enum {
   /* FILL ME */
 };
 
-#define CLASS(element)	GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element))
-
 static void			gst_element_class_init		(GstElementClass *klass);
 static void			gst_element_init		(GstElement *element);
 static void			gst_element_base_class_init	(GstElementClass *klass);
@@ -62,10 +60,6 @@ static void			gst_element_real_get_property	(GObject *object, guint prop_id, GVa
 
 static void 			gst_element_dispose 		(GObject *object);
 
-static gboolean 		gst_element_send_event_default 	(GstElement *element, GstEvent *event);
-static gboolean 		gst_element_query_default 	(GstElement *element, GstPadQueryType type,
-		          					 GstFormat *format, gint64 *value);
-
 static GstElementStateReturn	gst_element_change_state	(GstElement *element);
 static void			gst_element_error_func		(GstElement* element, GstElement *source, gchar *errormsg);
 
@@ -151,8 +145,6 @@ gst_element_class_init (GstElementClass *klass)
   klass->elementfactory 		= NULL;
   klass->padtemplates 			= NULL;
   klass->numpadtemplates 		= 0;
-  klass->send_event	 		= GST_DEBUG_FUNCPTR (gst_element_send_event_default);
-  klass->query		 		= GST_DEBUG_FUNCPTR (gst_element_query_default);
 }
 
 static void
@@ -186,7 +178,7 @@ gst_element_init (GstElement *element)
 static void
 gst_element_real_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-  GstElementClass *oclass = CLASS (object);
+  GstElementClass *oclass = GST_ELEMENT_GET_CLASS (object);
 
   if (oclass->set_property)
     (oclass->set_property) (object, prop_id, value, pspec);
@@ -195,7 +187,7 @@ gst_element_real_set_property (GObject *object, guint prop_id, const GValue *val
 static void
 gst_element_real_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-  GstElementClass *oclass = CLASS (object);
+  GstElementClass *oclass = GST_ELEMENT_GET_CLASS (object);
 
   if (oclass->get_property)
     (oclass->get_property) (object, prop_id, value, pspec);
@@ -641,7 +633,8 @@ gst_element_request_pad (GstElement *element, GstPadTemplate *templ, const gchar
   GstPad *newpad = NULL;
   GstElementClass *oclass;
 
-  oclass = CLASS (element);
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
   if (oclass->request_new_pad)
     newpad = (oclass->request_new_pad)(element, templ, name);
 
@@ -664,7 +657,8 @@ gst_element_release_request_pad (GstElement *element, GstPad *pad)
   g_return_if_fail (GST_IS_ELEMENT (element));
   g_return_if_fail (GST_IS_PAD (pad));
 
-  oclass = CLASS (element);
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
   if (oclass->release_pad)
     (oclass->release_pad) (element, pad);
 }
@@ -682,7 +676,7 @@ gst_element_requires_clock (GstElement *element)
 {
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  return (CLASS (element)->set_clock != NULL);
+  return (GST_ELEMENT_GET_CLASS (element)->set_clock != NULL);
 }
 
 /**
@@ -698,7 +692,7 @@ gst_element_provides_clock (GstElement *element)
 {
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  return (CLASS (element)->get_clock != NULL);
+  return (GST_ELEMENT_GET_CLASS (element)->get_clock != NULL);
 }
 
 /**
@@ -711,10 +705,14 @@ gst_element_provides_clock (GstElement *element)
 void
 gst_element_set_clock (GstElement *element, GstClock *clock)
 {
+  GstElementClass *oclass;
+
   g_return_if_fail (GST_IS_ELEMENT (element));
 
-  if (CLASS (element)->set_clock)
-    CLASS (element)->set_clock (element, clock);
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->set_clock)
+    oclass->set_clock (element, clock);
 
   element->clock = clock;
 }
@@ -730,11 +728,14 @@ gst_element_set_clock (GstElement *element, GstClock *clock)
 GstClock*
 gst_element_get_clock (GstElement *element)
 {
-  g_return_val_if_fail (element != NULL, NULL);
+  GstElementClass *oclass;
+
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
+
+  oclass = GST_ELEMENT_GET_CLASS (element);
   
-  if (CLASS (element)->get_clock)
-    return CLASS (element)->get_clock (element);
+  if (oclass->get_clock)
+    return oclass->get_clock (element);
 
   return NULL;
 }
@@ -742,8 +743,7 @@ gst_element_get_clock (GstElement *element)
 /**
  * gst_element_clock_wait:
  * @element: a #GstElement.
- * @clock: the #GstClock to use.
- * @time: the #GstClockTime to wait for on the clock.
+ * @id: the #GstClock to use.
  * @jitter: the difference between requested time and actual time.
  *
  * Waits for a specific time on the clock.
@@ -751,14 +751,14 @@ gst_element_get_clock (GstElement *element)
  * Returns: the #GstClockReturn result of the wait operation.
  */
 GstClockReturn
-gst_element_clock_wait (GstElement *element, GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter)
+gst_element_clock_wait (GstElement *element, GstClockID id, GstClockTimeDiff *jitter)
 {
   GstClockReturn res;
 
   g_return_val_if_fail (GST_IS_ELEMENT (element), GST_CLOCK_ERROR);
 
   if (GST_ELEMENT_SCHED (element)) {
-    res = gst_scheduler_clock_wait (GST_ELEMENT_SCHED (element), element, clock, time, jitter);
+    res = gst_scheduler_clock_wait (GST_ELEMENT_SCHED (element), element, id, jitter);
   }
   else 
     res = GST_CLOCK_TIMEOUT;
@@ -766,36 +766,67 @@ gst_element_clock_wait (GstElement *element, GstClock *clock, GstClockTime time,
   return res;
 }
 
+/**
+ * gst_element_is_indexable:
+ * @element: a #GstElement.
+ *
+ * Queries if the element can be indexed/
+ *
+ * Returns: TRUE if the element can be indexed.
+ */
 gboolean
-gst_element_is_cachable (GstElement *element)
+gst_element_is_indexable (GstElement *element)
 {
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  return (CLASS (element)->set_cache != NULL);
+  return (GST_ELEMENT_GET_CLASS (element)->set_index != NULL);
 }
 
+/**
+ * gst_element_set_index:
+ * @element: a #GstElement.
+ * @index: a #GstIndex.
+ *
+ * Set the specified GstIndex on the element.
+ */
 void
-gst_element_set_cache (GstElement *element, GstCache *cache)
+gst_element_set_index (GstElement *element, GstIndex *index)
 {
-  g_return_if_fail (GST_IS_ELEMENT (element));
-  g_return_if_fail (GST_IS_CACHE (cache));
+  GstElementClass *oclass;
 
-  if (CLASS (element)->set_cache)
-    CLASS (element)->set_cache (element, cache);
+  g_return_if_fail (GST_IS_ELEMENT (element));
+  g_return_if_fail (GST_IS_INDEX (index));
+
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->set_index)
+    oclass->set_index (element, index);
 }
 
-GstCache*
-gst_element_get_cache (GstElement *element)
+/**
+ * gst_element_get_index:
+ * @element: a #GstElement.
+ *
+ * Gets the index from the element.
+ *
+ * Returns: a #GstIndex or NULL when no index was set on the
+ * element.
+ */
+GstIndex*
+gst_element_get_index (GstElement *element)
 {
+  GstElementClass *oclass;
+
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  if (CLASS (element)->get_cache)
-    return CLASS (element)->get_cache (element);
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->get_index)
+    return oclass->get_index (element);
 
   return NULL;
 }
 
-
 /**
  * gst_element_release_locks:
  * @element: a #GstElement to release all locks on.
@@ -808,10 +839,14 @@ gst_element_get_cache (GstElement *element)
 gboolean
 gst_element_release_locks (GstElement *element)
 {
+  GstElementClass *oclass;
+
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  if (CLASS (element)->release_locks)
-    return CLASS (element)->release_locks (element);
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->release_locks)
+    return oclass->release_locks (element);
   
   return TRUE;
 }
@@ -1148,7 +1183,7 @@ gst_element_get_pad_template_list (GstElement *element)
   g_return_val_if_fail (element != NULL, NULL);
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
 
-  return CLASS (element)->padtemplates;
+  return GST_ELEMENT_GET_CLASS (element)->padtemplates;
 }
 
 /**
@@ -1721,24 +1756,65 @@ gst_element_error_func (GstElement* element, GstElement *source,
   }
 }
 
-static gboolean
-gst_element_send_event_default (GstElement *element, GstEvent *event)
+static GstPad*
+gst_element_get_random_pad (GstElement *element, GstPadDirection dir)
 {
   GList *pads = element->pads;
-  gboolean res = FALSE;
-
+  GST_DEBUG (GST_CAT_ELEMENT_PADS, "getting a random pad");
   while (pads) {
     GstPad *pad = GST_PAD_CAST (pads->data);
-    
-    if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK) {
-      if (GST_PAD_IS_USABLE (pad)) {
-	res = gst_pad_send_event (GST_PAD_PEER (pad), event);
-	break;
+
+    GST_DEBUG (GST_CAT_ELEMENT_PADS, "checking pad %s:%s",
+	       GST_DEBUG_PAD_NAME (pad));
+
+    if (GST_PAD_DIRECTION (pad) == dir) {
+      if (GST_PAD_IS_CONNECTED (pad)) {
+	return pad;
+      }
+      else {
+        GST_DEBUG (GST_CAT_ELEMENT_PADS, "pad %s:%s is not connected",
+	           GST_DEBUG_PAD_NAME (pad));
       }
     }
+    else {
+      GST_DEBUG (GST_CAT_ELEMENT_PADS, "pad %s:%s is in wrong direction",
+                 GST_DEBUG_PAD_NAME (pad));
+    }
+
     pads = g_list_next (pads);
   }
-  return res;
+  return NULL;
+}
+
+/**
+ * gst_element_get_event_masks:
+ * @element: a #GstElement to query
+ *
+ * Get an array of event masks from the element.
+ * If the element doesn't 
+ * implement an event masks function, the query will be forwarded
+ * to a random connected sink pad.
+ * 
+ * Returns: An array of #GstEventMask elements.
+ */
+const GstEventMask*
+gst_element_get_event_masks (GstElement *element)
+{
+  GstElementClass *oclass;
+
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->get_event_masks)
+    return oclass->get_event_masks (element);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad)
+      return gst_pad_get_event_masks (GST_PAD_PEER (pad));
+  }
+
+  return FALSE;
 }
 
 /**
@@ -1746,49 +1822,72 @@ gst_element_send_event_default (GstElement *element, GstEvent *event)
  * @element: a #GstElement to send the event to.
  * @event: the #GstEvent to send to the element.
  *
- * Sends an event to an element. If the element doesn't 
+ * Sends an event to an element. If the element doesn't
  * implement an event handler, the event will be forwarded
  * to a random sink pad.
- * 
+ *
  * Returns: TRUE if the event was handled.
  */
 gboolean
 gst_element_send_event (GstElement *element, GstEvent *event)
 {
+  GstElementClass *oclass;
+
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
-  if (CLASS (element)->send_event)
-    return CLASS (element)->send_event (element, event);
+  oclass = GST_ELEMENT_GET_CLASS (element);
 
+  if (oclass->send_event)
+    return oclass->send_event (element, event);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad) {
+      GST_DEBUG (GST_CAT_ELEMENT_PADS, "sending event to random pad %s:%s",
+		 GST_DEBUG_PAD_NAME (pad));
+      return gst_pad_send_event (GST_PAD_PEER (pad), event);
+    }
+  }
+  GST_DEBUG (GST_CAT_ELEMENT_PADS, "can't send event on element %s",
+	     GST_ELEMENT_NAME (element));
   return FALSE;
 }
 
-static gboolean
-gst_element_query_default (GstElement *element, GstPadQueryType type,
-		           GstFormat *format, gint64 *value)
+/**
+ * gst_element_get_query_types:
+ * @element: a #GstElement to query
+ *
+ * Get an array of query types from the element.
+ * If the element doesn't 
+ * implement a query types function, the query will be forwarded
+ * to a random sink pad.
+ * 
+ * Returns: An array of #GstQueryType elements.
+ */
+const GstQueryType*
+gst_element_get_query_types (GstElement *element)
 {
-  GList *pads = element->pads;
-  gboolean res = FALSE;
+  GstElementClass *oclass;
 
-  while (pads) {
-    GstPad *pad = GST_PAD_CAST (pads->data);
-    
-    if (GST_PAD_DIRECTION (pad) == GST_PAD_SINK) {
-      if (GST_PAD_IS_USABLE (pad)) {
-	res = gst_pad_query (GST_PAD_PEER (pad), type, format, value);
-	break;
-      }
-    }
-    pads = g_list_next (pads);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->get_query_types)
+    return oclass->get_query_types (element);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad)
+      return gst_pad_get_query_types (GST_PAD_PEER (pad));
   }
-  return res;
+
+  return FALSE;
 }
 
 /**
  * gst_element_query:
  * @element: a #GstElement to perform the query on.
- * @type: the #GstPadQueryType.
+ * @type: the #GstQueryType.
  * @format: the #GstFormat pointer to hold the format of the result.
  * @value: the pointer to the value of the result.
  *
@@ -1796,20 +1895,108 @@ gst_element_query_default (GstElement *element, GstPadQueryType type,
  * to GST_FORMAT_DEFAULT and this function returns TRUE, the 
  * format pointer will hold the default format.
  * For element that don't implement a query handler, this function
- * forwards the query to a random connected sinkpad of this element.
+ * forwards the query to a random usable sinkpad of this element.
  * 
  * Returns: TRUE if the query could be performed.
  */
 gboolean
-gst_element_query (GstElement *element, GstPadQueryType type,
+gst_element_query (GstElement *element, GstQueryType type,
 		   GstFormat *format, gint64 *value)
 {
+  GstElementClass *oclass;
+
   g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
   g_return_val_if_fail (format != NULL, FALSE);
   g_return_val_if_fail (value != NULL, FALSE);
+
+  oclass = GST_ELEMENT_GET_CLASS (element);
   
-  if (CLASS (element)->query)
-    return CLASS (element)->query (element, type, format, value);
+  if (oclass->query)
+    return oclass->query (element, type, format, value);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad)
+      return gst_pad_query (GST_PAD_PEER (pad), type, format, value);
+  }
+
+  return FALSE;
+}
+
+/**
+ * gst_element_get_formats:
+ * @element: a #GstElement to query
+ *
+ * Get an array of formst from the element.
+ * If the element doesn't 
+ * implement a formats function, the query will be forwarded
+ * to a random sink pad.
+ * 
+ * Returns: An array of #GstFormat elements.
+ */
+const GstFormat*
+gst_element_get_formats (GstElement *element)
+{
+  GstElementClass *oclass;
+
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->get_formats)
+    return oclass->get_formats (element);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad)
+      return gst_pad_get_formats (GST_PAD_PEER (pad));
+  }
+
+  return FALSE;
+}
+
+/**
+ * gst_element_convert:
+ * @element: a #GstElement to invoke the converter on.
+ * @src_format: the source #GstFormat.
+ * @src_value: the source value.
+ * @dest_format: a pointer to the destination #GstFormat.
+ * @dest_value: a pointer to the destination value.
+ *
+ * Invokes a conversion on the element.
+ * If the element doesn't 
+ * implement a convert function, the query will be forwarded
+ * to a random sink pad.
+ *
+ * Returns: TRUE if the conversion could be performed.
+ */
+gboolean
+gst_element_convert (GstElement *element,
+		     GstFormat  src_format,  gint64  src_value,
+		     GstFormat *dest_format, gint64 *dest_value)
+{
+  GstElementClass *oclass;
+
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (dest_format != NULL, FALSE);
+  g_return_val_if_fail (dest_value != NULL, FALSE);
+
+  if (src_format == *dest_format) {
+    *dest_value = src_value;
+    return TRUE;
+  }
+
+  oclass = GST_ELEMENT_GET_CLASS (element);
+
+  if (oclass->convert)
+    return oclass->convert (element,
+		            src_format, src_value, 
+		            dest_format, dest_value);
+  else {
+    GstPad *pad = gst_element_get_random_pad (element, GST_PAD_SINK);
+    if (pad)
+      return gst_pad_convert (GST_PAD_PEER (pad), 
+		              src_format, src_value, 
+		              dest_format, dest_value);
+  }
 
   return FALSE;
 }
@@ -1849,6 +2036,9 @@ gst_element_error (GstElement *element, const gchar *error, ...)
     gst_scheduler_error (element->sched, element); 
   } 
 
+  if (GST_STATE (element) == GST_STATE_PLAYING)
+    gst_element_set_state (element, GST_STATE_PAUSED);
+
   /* cleanup */
   gst_object_unref (GST_OBJECT (element));
   g_free (string);
@@ -1933,7 +2123,7 @@ gst_element_set_state (GstElement *element, GstElementState state)
     }
 
     /* call the state change function so it can set the state */
-    oclass = CLASS (element);
+    oclass = GST_ELEMENT_GET_CLASS (element);
     if (oclass->change_state)
       return_val = (oclass->change_state) (element);
 
@@ -1989,7 +2179,7 @@ gst_element_negotiate_pads (GstElement *element)
 
     /* if we have a connection on this pad and it doesn't have caps
      * allready, try to negotiate */
-    if (GST_PAD_IS_USABLE (srcpad) && !GST_PAD_CAPS (srcpad)) {
+    if (GST_PAD_IS_CONNECTED (srcpad) && !GST_PAD_CAPS (srcpad)) {
       GstRealPad *sinkpad;
       GstElementState otherstate;
       GstElement *parent;
@@ -2001,6 +2191,11 @@ gst_element_negotiate_pads (GstElement *element)
       if (!parent) 
 	continue;
 
+      /* skips pads that were already negotiating */
+      if (GST_FLAG_IS_SET (sinkpad, GST_PAD_NEGOTIATING) ||
+          GST_FLAG_IS_SET (srcpad, GST_PAD_NEGOTIATING))
+	continue;
+
       otherstate = GST_STATE (parent);
 
       /* swap pads if needed */
@@ -2050,6 +2245,22 @@ gst_element_clear_pad_caps (GstElement *element)
   }
 }
 
+static void
+gst_element_pads_activate (GstElement *element, gboolean active)
+{
+  GList *pads = element->pads;
+
+  while (pads) {
+    GstPad *pad = GST_PAD_CAST (pads->data);
+    pads = g_list_next (pads);
+
+    if (!GST_IS_REAL_PAD (pad))
+      continue;
+
+    gst_pad_set_active (pad, active);
+  }
+}
+
 static GstElementStateReturn
 gst_element_change_state (GstElement *element)
 {
@@ -2071,17 +2282,23 @@ gst_element_change_state (GstElement *element)
     return GST_STATE_SUCCESS;
   }
   
-  GST_INFO (GST_CAT_STATES, "%s default handler sets state from %s to %s %d", 
+  GST_INFO (GST_CAT_STATES, "%s default handler sets state from %s to %s %04x", 
             GST_ELEMENT_NAME (element),
             gst_element_state_get_name (old_state),
             gst_element_state_get_name (old_pending),
-	    GST_STATE_TRANSITION (element));
+	    old_transition);
 
   /* we set the state change early for the negotiation functions */
   GST_STATE (element) = old_pending;
   GST_STATE_PENDING (element) = GST_STATE_VOID_PENDING;
 
   switch (old_transition) {
+    case GST_STATE_PLAYING_TO_PAUSED:
+      gst_element_pads_activate (element, FALSE);
+      break;
+    case GST_STATE_PAUSED_TO_PLAYING:
+      gst_element_pads_activate (element, TRUE);
+      break;
     /* if we are going to paused, we try to negotiate the pads */
     case GST_STATE_READY_TO_PAUSED:
       if (!gst_element_negotiate_pads (element)) 
@@ -2145,13 +2362,9 @@ failure:
 GstElementFactory*
 gst_element_get_factory (GstElement *element)
 {
-  GstElementClass *oclass;
-
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
 
-  oclass = CLASS (element);
-
-  return oclass->elementfactory;
+  return GST_ELEMENT_GET_CLASS (element)->elementfactory;
 }
 
 static void
@@ -2226,7 +2439,7 @@ gst_element_save_thyself (GstObject *object,
 
   element = GST_ELEMENT (object);
 
-  oclass = CLASS (element);
+  oclass = GST_ELEMENT_GET_CLASS (element);
 
   xmlNewChild(parent, NULL, "name", GST_ELEMENT_NAME(element));
 
@@ -2329,8 +2542,8 @@ gst_element_restore_thyself (GstObject *object, xmlNodePtr self)
     children = children->next;
   }
 
-  if (GST_OBJECT_CLASS(parent_class)->restore_thyself)
-    (GST_OBJECT_CLASS(parent_class)->restore_thyself) (object, self);
+  if (GST_OBJECT_CLASS (parent_class)->restore_thyself)
+    (GST_OBJECT_CLASS (parent_class)->restore_thyself) (object, self);
 }
 #endif /* GST_DISABLE_LOADSAVE */
 
@@ -2428,6 +2641,10 @@ gst_element_set_loop_function (GstElement *element,
 
   /* set the NEW_LOOPFUNC flag so everyone knows to go try again */
   GST_FLAG_SET (element, GST_ELEMENT_NEW_LOOPFUNC);
+
+  if (GST_ELEMENT_SCHED (element)) {
+    gst_scheduler_scheduling_change (GST_ELEMENT_SCHED (element), element);
+  }
 }
 
 /**
