@@ -135,6 +135,37 @@ gst_buffer_flag_get_type (void)
 
 
 /* enumerations from "gstclock.h" */
+GType
+gst_clock_entry_status_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_CLOCK_ENTRY_OK, "GST_CLOCK_ENTRY_OK", "ok" },
+      { GST_CLOCK_ENTRY_EARLY, "GST_CLOCK_ENTRY_EARLY", "early" },
+      { GST_CLOCK_ENTRY_RESTART, "GST_CLOCK_ENTRY_RESTART", "restart" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstClockEntryStatus", values);
+  }
+  return etype;
+}
+
+GType
+gst_clock_entry_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_CLOCK_ENTRY_SINGLE, "GST_CLOCK_ENTRY_SINGLE", "single" },
+      { GST_CLOCK_ENTRY_PERIODIC, "GST_CLOCK_ENTRY_PERIODIC", "periodic" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstClockEntryType", values);
+  }
+  return etype;
+}
+
 GType
 gst_clock_return_get_type (void)
 {
@@ -145,6 +176,7 @@ gst_clock_return_get_type (void)
       { GST_CLOCK_TIMEOUT, "GST_CLOCK_TIMEOUT", "timeout" },
       { GST_CLOCK_EARLY, "GST_CLOCK_EARLY", "early" },
       { GST_CLOCK_ERROR, "GST_CLOCK_ERROR", "error" },
+      { GST_CLOCK_UNSUPPORTED, "GST_CLOCK_UNSUPPORTED", "unsupported" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstClockReturn", values);
@@ -152,6 +184,25 @@ gst_clock_return_get_type (void)
   return etype;
 }
 
+GType
+gst_clock_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_CLOCK_FLAG_CAN_DO_SINGLE_SYNC, "GST_CLOCK_FLAG_CAN_DO_SINGLE_SYNC", "do-single-sync" },
+      { GST_CLOCK_FLAG_CAN_DO_SINGLE_ASYNC, "GST_CLOCK_FLAG_CAN_DO_SINGLE_ASYNC", "do-single-async" },
+      { GST_CLOCK_FLAG_CAN_DO_PERIODIC_SYNC, "GST_CLOCK_FLAG_CAN_DO_PERIODIC_SYNC", "do-periodic-sync" },
+      { GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC, "GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC", "do-periodic-async" },
+      { GST_CLOCK_FLAG_CAN_SET_RESOLUTION, "GST_CLOCK_FLAG_CAN_SET_RESOLUTION", "set-resolution" },
+      { GST_CLOCK_FLAG_CAN_SET_SPEED, "GST_CLOCK_FLAG_CAN_SET_SPEED", "set-speed" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstClockFlags", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gstcpu.h" */
 GType
@@ -234,6 +285,7 @@ gst_event_type_get_type (void)
       { GST_EVENT_SIZE, "GST_EVENT_SIZE", "size" },
       { GST_EVENT_RATE, "GST_EVENT_RATE", "rate" },
       { GST_EVENT_FILLER, "GST_EVENT_FILLER", "filler" },
+      { GST_EVENT_TS_OFFSET, "GST_EVENT_TS_OFFSET", "ts-offset" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstEventType", values);
@@ -314,6 +366,105 @@ gst_format_get_type (void)
 }
 
 
+/* enumerations from "gstindex.h" */
+GType
+gst_index_certainty_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_INDEX_UNKNOWN, "GST_INDEX_UNKNOWN", "unknown" },
+      { GST_INDEX_CERTAIN, "GST_INDEX_CERTAIN", "certain" },
+      { GST_INDEX_FUZZY, "GST_INDEX_FUZZY", "fuzzy" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstIndexCertainty", values);
+  }
+  return etype;
+}
+
+GType
+gst_index_entry_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_INDEX_ENTRY_ID, "GST_INDEX_ENTRY_ID", "id" },
+      { GST_INDEX_ENTRY_ASSOCIATION, "GST_INDEX_ENTRY_ASSOCIATION", "association" },
+      { GST_INDEX_ENTRY_OBJECT, "GST_INDEX_ENTRY_OBJECT", "object" },
+      { GST_INDEX_ENTRY_FORMAT, "GST_INDEX_ENTRY_FORMAT", "format" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstIndexEntryType", values);
+  }
+  return etype;
+}
+
+GType
+gst_index_lookup_method_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_INDEX_LOOKUP_EXACT, "GST_INDEX_LOOKUP_EXACT", "exact" },
+      { GST_INDEX_LOOKUP_BEFORE, "GST_INDEX_LOOKUP_BEFORE", "before" },
+      { GST_INDEX_LOOKUP_AFTER, "GST_INDEX_LOOKUP_AFTER", "after" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstIndexLookupMethod", values);
+  }
+  return etype;
+}
+
+GType
+gst_assoc_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { GST_ASSOCIATION_FLAG_NONE, "GST_ASSOCIATION_FLAG_NONE", "none" },
+      { GST_ASSOCIATION_FLAG_KEY_UNIT, "GST_ASSOCIATION_FLAG_KEY_UNIT", "key-unit" },
+      { GST_ASSOCIATION_FLAG_LAST, "GST_ASSOCIATION_FLAG_LAST", "last" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("GstAssocFlags", values);
+  }
+  return etype;
+}
+
+GType
+gst_index_resolver_method_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_INDEX_RESOLVER_CUSTOM, "GST_INDEX_RESOLVER_CUSTOM", "custom" },
+      { GST_INDEX_RESOLVER_GTYPE, "GST_INDEX_RESOLVER_GTYPE", "gtype" },
+      { GST_INDEX_RESOLVER_PATH, "GST_INDEX_RESOLVER_PATH", "path" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstIndexResolverMethod", values);
+  }
+  return etype;
+}
+
+GType
+gst_index_flags_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_INDEX_WRITABLE, "GST_INDEX_WRITABLE", "writable" },
+      { GST_INDEX_READABLE, "GST_INDEX_READABLE", "readable" },
+      { GST_INDEX_FLAG_LAST, "GST_INDEX_FLAG_LAST", "flag-last" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstIndexFlags", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstpad.h" */
 GType
 gst_pad_connect_return_get_type (void)
@@ -332,27 +483,6 @@ gst_pad_connect_return_get_type (void)
   return etype;
 }
 
-GType
-gst_pad_query_type_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { GST_PAD_QUERY_NONE, "GST_PAD_QUERY_NONE", "none" },
-      { GST_PAD_QUERY_TOTAL, "GST_PAD_QUERY_TOTAL", "total" },
-      { GST_PAD_QUERY_POSITION, "GST_PAD_QUERY_POSITION", "position" },
-      { GST_PAD_QUERY_LATENCY, "GST_PAD_QUERY_LATENCY", "latency" },
-      { GST_PAD_QUERY_JITTER, "GST_PAD_QUERY_JITTER", "jitter" },
-      { GST_PAD_QUERY_START, "GST_PAD_QUERY_START", "start" },
-      { GST_PAD_QUERY_SEGMENT_END, "GST_PAD_QUERY_SEGMENT_END", "segment-end" },
-      { GST_PAD_QUERY_RATE, "GST_PAD_QUERY_RATE", "rate" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("GstPadQueryType", values);
-  }
-  return etype;
-}
-
 GType
 gst_pad_direction_get_type (void)
 {
@@ -376,7 +506,7 @@ gst_pad_flags_get_type (void)
   if (etype == 0) {
     static const GEnumValue values[] = {
       { GST_PAD_DISABLED, "GST_PAD_DISABLED", "disabled" },
-      { GST_PAD_EOS, "GST_PAD_EOS", "eos" },
+      { GST_PAD_NEGOTIATING, "GST_PAD_NEGOTIATING", "negotiating" },
       { GST_PAD_FLAG_LAST, "GST_PAD_FLAG_LAST", "flag-last" },
       { 0, NULL, NULL }
     };
@@ -431,7 +561,7 @@ gst_props_type_get_type (void)
       { GST_PROPS_INT_TYPE, "GST_PROPS_INT_TYPE", "int-type" },
       { GST_PROPS_FLOAT_TYPE, "GST_PROPS_FLOAT_TYPE", "float-type" },
       { GST_PROPS_FOURCC_TYPE, "GST_PROPS_FOURCC_TYPE", "fourcc-type" },
-      { GST_PROPS_BOOL_TYPE, "GST_PROPS_BOOL_TYPE", "bool-type" },
+      { GST_PROPS_BOOLEAN_TYPE, "GST_PROPS_BOOLEAN_TYPE", "boolean-type" },
       { GST_PROPS_STRING_TYPE, "GST_PROPS_STRING_TYPE", "string-type" },
       { GST_PROPS_VAR_TYPE, "GST_PROPS_VAR_TYPE", "var-type" },
       { GST_PROPS_LIST_TYPE, "GST_PROPS_LIST_TYPE", "list-type" },
@@ -447,6 +577,29 @@ gst_props_type_get_type (void)
 }
 
 
+/* enumerations from "gstquery.h" */
+GType
+gst_query_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_QUERY_NONE, "GST_QUERY_NONE", "none" },
+      { GST_QUERY_TOTAL, "GST_QUERY_TOTAL", "total" },
+      { GST_QUERY_POSITION, "GST_QUERY_POSITION", "position" },
+      { GST_QUERY_LATENCY, "GST_QUERY_LATENCY", "latency" },
+      { GST_QUERY_JITTER, "GST_QUERY_JITTER", "jitter" },
+      { GST_QUERY_START, "GST_QUERY_START", "start" },
+      { GST_QUERY_SEGMENT_END, "GST_QUERY_SEGMENT_END", "segment-end" },
+      { GST_QUERY_RATE, "GST_QUERY_RATE", "rate" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstQueryType", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstscheduler.h" */
 GType
 gst_scheduler_flags_get_type (void)
@@ -538,26 +691,6 @@ gst_registry_flags_get_type (void)
 }
 
 
-/* enumerations from "gsttimecache.h" */
-GType
-gst_time_cache_certainty_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { GST_TIME_CACHE_UNKNOWN, "GST_TIME_CACHE_UNKNOWN", "unknown" },
-      { GST_TIME_CACHE_CERTAIN, "GST_TIME_CACHE_CERTAIN", "certain" },
-      { GST_TIME_CACHE_FUZZY_LOCATION, "GST_TIME_CACHE_FUZZY_LOCATION", "fuzzy-location" },
-      { GST_TIME_CACHE_FUZZY_TIMESTAMP, "GST_TIME_CACHE_FUZZY_TIMESTAMP", "fuzzy-timestamp" },
-      { GST_TIME_CACHE_FUZZY, "GST_TIME_CACHE_FUZZY", "fuzzy" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("GstTimeCacheCertainty", values);
-  }
-  return etype;
-}
-
-
 /* enumerations from "gstparse.h" */
 GType
 gst_parse_error_get_type (void)
