@@ -30,6 +30,8 @@
 #include <gst/gstbuffer.h>
 #include <gst/gstcaps.h>
 #include <gst/gstevent.h>
+#include <gst/gstprobe.h>
+#include <gst/gstquery.h>
 
 
 G_BEGIN_DECLS
@@ -50,7 +52,7 @@ extern GType _gst_ghost_pad_type;
 #define GST_IS_PAD(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PAD))
 #define GST_IS_PAD_FAST(obj)		(G_OBJECT_TYPE(obj) == GST_TYPE_REAL_PAD || \
 					 G_OBJECT_TYPE(obj) == GST_TYPE_GHOST_PAD)
-#define GST_IS_PAD_CLASS(obj)		(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PAD))
+#define GST_IS_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PAD))
 
 #ifdef GST_TYPE_PARANOID
 # define GST_PAD(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PAD, GstPad))
@@ -69,7 +71,7 @@ extern GType _gst_ghost_pad_type;
 #define GST_REAL_PAD_CLASS_CAST(klass)	((GstRealPadClass*)(klass))
 #define GST_IS_REAL_PAD(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_REAL_PAD))
 #define GST_IS_REAL_PAD_FAST(obj)	(G_OBJECT_TYPE(obj) == GST_TYPE_REAL_PAD)
-#define GST_IS_REAL_PAD_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_REAL_PAD))
+#define GST_IS_REAL_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_REAL_PAD))
 
 #ifdef GST_TYPE_PARANOID
 # define GST_REAL_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_REAL_PAD, GstRealPad))
@@ -88,7 +90,7 @@ extern GType _gst_ghost_pad_type;
 #define GST_GHOST_PAD_CLASS_CAST(klass)	((GstGhostPadClass*)(klass))
 #define GST_IS_GHOST_PAD(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_GHOST_PAD))
 #define GST_IS_GHOST_PAD_FAST(obj)	(G_OBJECT_TYPE(obj) == GST_TYPE_GHOST_PAD)
-#define GST_IS_GHOST_PAD_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_GHOST_PAD))
+#define GST_IS_GHOST_PAD_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_GHOST_PAD))
 
 #ifdef GST_TYPE_PARANOID
 # define GST_GHOST_PAD(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_GHOST_PAD, GstGhostPad))
@@ -116,27 +118,16 @@ typedef enum {
   GST_PAD_CONNECT_DONE    =  2
 } GstPadConnectReturn;
 
-typedef enum {
-  GST_PAD_QUERY_NONE = 0,
-  GST_PAD_QUERY_TOTAL,
-  GST_PAD_QUERY_POSITION,
-  GST_PAD_QUERY_LATENCY,
-  GST_PAD_QUERY_JITTER,
-  GST_PAD_QUERY_START,
-  GST_PAD_QUERY_SEGMENT_END,
-  GST_PAD_QUERY_RATE
-} GstPadQueryType;
-
-#define GST_PAD_QUERY_TYPE_FUNCTION(functionname, a...) \
-static const GstPadQueryType*                           \
-functionname (GstPad *pad)                              \
-{                                                       \
-  static const GstPadQueryType types[] = {              \
-    a,                                                  \
-    0                                              	\
-  };                                                    \
-  return types;                                         \
-}
+/* convenience functions */
+#ifdef G_HAVE_ISO_VARARGS
+#define GST_PAD_QUERY_TYPE_FUNCTION(functionname, ...)  GST_QUERY_TYPE_FUNCTION (GstPad *, functionname, __VA_ARGS__);
+#define GST_PAD_FORMATS_FUNCTION(functionname, ...)  	GST_FORMATS_FUNCTION (GstPad *, functionname, __VA_ARGS__);
+#define GST_PAD_EVENT_MASK_FUNCTION(functionname, ...) 	GST_EVENT_MASK_FUNCTION (GstPad *, functionname, __VA_ARGS__);
+#elif defined(G_HAVE_GNUC_VARARGS)
+#define GST_PAD_QUERY_TYPE_FUNCTION(functionname, a...) GST_QUERY_TYPE_FUNCTION (GstPad *, functionname, a);
+#define GST_PAD_FORMATS_FUNCTION(functionname, a...)  	GST_FORMATS_FUNCTION (GstPad *, functionname, a);
+#define GST_PAD_EVENT_MASK_FUNCTION(functionname, a...) GST_EVENT_MASK_FUNCTION (GstPad *, functionname, a);
+#endif
 
  
 /* this defines the functions used to chain buffers
@@ -148,12 +139,12 @@ typedef gboolean		(*GstPadEventFunction)		(GstPad *pad, GstEvent *event);
 typedef gboolean		(*GstPadConvertFunction)	(GstPad *pad, 
 		 						 GstFormat src_format,  gint64  src_value,
 								 GstFormat *dest_format, gint64 *dest_value);
-typedef gboolean		(*GstPadQueryFunction)		(GstPad *pad, GstPadQueryType type,
+typedef gboolean		(*GstPadQueryFunction)		(GstPad *pad, GstQueryType type,
 		 						 GstFormat *format, gint64  *value);
 typedef GList*			(*GstPadIntConnFunction)	(GstPad *pad);
 typedef const GstFormat*	(*GstPadFormatsFunction)	(GstPad *pad);
 typedef const GstEventMask*	(*GstPadEventMaskFunction)	(GstPad *pad);
-typedef const GstPadQueryType*	(*GstPadQueryTypeFunction)	(GstPad *pad);
+typedef const GstQueryType*	(*GstPadQueryTypeFunction)	(GstPad *pad);
 
 typedef GstPadConnectReturn	(*GstPadConnectFunction) 	(GstPad *pad, GstCaps *caps);
 typedef GstCaps*		(*GstPadGetCapsFunction) 	(GstPad *pad, GstCaps *caps);
@@ -169,7 +160,7 @@ typedef enum {
 
 typedef enum {
   GST_PAD_DISABLED		= GST_OBJECT_FLAG_LAST,
-  GST_PAD_EOS,
+  GST_PAD_NEGOTIATING,
 
   GST_PAD_FLAG_LAST		= GST_OBJECT_FLAG_LAST + 4
 } GstPadFlags;
@@ -221,20 +212,17 @@ struct _GstRealPad {
   GstPadIntConnFunction		 intconnfunc;
 
   GstPadBufferPoolFunction 	 bufferpoolfunc;
+
+  GstProbeDispatcher 		 probedisp;
 };
 
 struct _GstRealPadClass {
   GstPadClass parent_class;
 
   /* signal callbacks */
-  void (*set_active)		(GstPad *pad, gboolean active);
-  void (*caps_changed)		(GstPad *pad, GstCaps *newcaps);
   void (*caps_nego_failed)	(GstPad *pad);
   void (*connected)		(GstPad *pad, GstPad *peer);
   void (*disconnected)		(GstPad *pad, GstPad *peer);
-  void (*event_received)	(GstPad *pad, GstEvent *event);
-
-  void (*eos)			(GstPad *pad);
 };
 
 struct _GstGhostPad {
@@ -274,9 +262,6 @@ struct _GstGhostPadClass {
 #define GST_RPAD_QUERYTYPEFUNC(pad)	(((GstRealPad *)(pad))->querytypefunc)
 #define GST_RPAD_EVENTMASKFUNC(pad)	(((GstRealPad *)(pad))->eventmaskfunc)
 
-#define GST_RPAD_EVENT_MASKS(pad)	(((GstRealPad *)(pad))->event_masks)
-#define GST_RPAD_QUERY_TYPES(pad)	(((GstRealPad *)(pad))->query_types)
-
 #define GST_RPAD_CONNECTFUNC(pad)	(((GstRealPad *)(pad))->connectfunc)
 #define GST_RPAD_GETCAPSFUNC(pad)	(((GstRealPad *)(pad))->getcapsfunc)
 #define GST_RPAD_BUFFERPOOLFUNC(pad)	(((GstRealPad *)(pad))->bufferpoolfunc)
@@ -292,7 +277,7 @@ struct _GstGhostPadClass {
 
 /* Some check functions (unused?) */
 #define GST_PAD_IS_CONNECTED(pad)	(GST_PAD_PEER(pad) != NULL)
-#define GST_PAD_IS_ACTIVE(pad)		(!GST_FLAG_IS_SET(pad, GST_PAD_DISABLED))
+#define GST_PAD_IS_ACTIVE(pad)		(!GST_FLAG_IS_SET(GST_PAD_REALIZE(pad), GST_PAD_DISABLED))
 #define GST_PAD_IS_USABLE(pad)		(GST_PAD_IS_CONNECTED (pad) && \
 		                         GST_PAD_IS_ACTIVE(pad) && GST_PAD_IS_ACTIVE(GST_PAD_PEER (pad)))
 #define GST_PAD_CAN_PULL(pad)		(GST_IS_REAL_PAD(pad) && GST_REAL_PAD(pad)->gethandler != NULL)
@@ -304,7 +289,7 @@ struct _GstGhostPadClass {
 #define GST_PAD_TEMPLATE(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PAD_TEMPLATE,GstPadTemplate))
 #define GST_PAD_TEMPLATE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PAD_TEMPLATE,GstPadTemplateClass))
 #define GST_IS_PAD_TEMPLATE(obj)	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PAD_TEMPLATE))
-#define GST_IS_PAD_TEMPLATE_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PAD_TEMPLATE))
+#define GST_IS_PAD_TEMPLATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PAD_TEMPLATE))
 
 typedef enum {
   GST_PAD_ALWAYS,
@@ -337,6 +322,30 @@ struct _GstPadTemplateClass {
   void (*pad_created)	(GstPadTemplate *templ, GstPad *pad);
 };
 
+#ifdef G_HAVE_ISO_VARARGS
+#define GST_PAD_TEMPLATE_NEW(padname, dir, pres, ...) \
+  gst_pad_template_new (                        \
+    padname,                                    \
+    dir,                                        \
+    pres,                                       \
+    __VA_ARGS__ ,				\
+    NULL)
+
+#define GST_PAD_TEMPLATE_FACTORY(name, padname, dir, pres, ...) \
+static GstPadTemplate*                          \
+name (void)                                     \
+{                                               \
+  static GstPadTemplate *templ = NULL;       	\
+  if (!templ) {                              	\
+    templ = GST_PAD_TEMPLATE_NEW (            	\
+      padname,                          	\
+      dir,                                      \
+      pres,                                     \
+      __VA_ARGS__ );                            \
+  }                                             \
+  return templ;                              	\
+}
+#elif defined(G_HAVE_GNUC_VARARGS)
 /* CR1: the space after 'a' is necessary because of preprocessing in gcc */
 #define GST_PAD_TEMPLATE_NEW(padname, dir, pres, a...) \
   gst_pad_template_new (                        \
@@ -360,6 +369,7 @@ name (void)                                     \
   }                                             \
   return templ;                              	\
 }
+#endif
 
 #define GST_PAD_TEMPLATE_GET(fact) (fact)()
 
@@ -407,7 +417,6 @@ void			gst_pad_set_event_function		(GstPad *pad, GstPadEventFunction event);
 void			gst_pad_set_event_mask_function		(GstPad *pad, GstPadEventMaskFunction mask_func);
 const GstEventMask*	gst_pad_get_event_masks			(GstPad *pad);
 const GstEventMask*	gst_pad_get_event_masks_default		(GstPad *pad);
-gboolean		gst_pad_handles_event			(GstPad *pad, GstEventMask *mask);
 
 /* pad connections */
 void			gst_pad_set_connect_function		(GstPad *pad, GstPadConnectFunction connect);
@@ -444,8 +453,8 @@ GstPad*			gst_pad_selectv				(GstPad *pad, ...);
 
 
 /* convert/query/format functions */
-void			gst_pad_set_formats_function		(GstPad *pad, GstPadFormatsFunction format);
-gboolean		gst_pad_handles_format			(GstPad *pad, GstFormat format);
+void			gst_pad_set_formats_function		(GstPad *pad, 
+								 GstPadFormatsFunction formats);
 const GstFormat*	gst_pad_get_formats			(GstPad *pad);
 const GstFormat*	gst_pad_get_formats_default		(GstPad *pad);
 
@@ -458,12 +467,12 @@ gboolean		gst_pad_convert_default 		(GstPad *pad,
 					                         GstFormat *dest_format, gint64 *dest_value);
 
 void			gst_pad_set_query_function		(GstPad *pad, GstPadQueryFunction query);
-void			gst_pad_set_query_type_function		(GstPad *pad, GstPadQueryTypeFunction type_function);
-const GstPadQueryType*	gst_pad_get_query_types			(GstPad *pad);
-const GstPadQueryType*	gst_pad_get_query_types_default		(GstPad *pad);
-gboolean		gst_pad_query				(GstPad *pad, GstPadQueryType type,
+void			gst_pad_set_query_type_function		(GstPad *pad, GstPadQueryTypeFunction type_func);
+const GstQueryType*	gst_pad_get_query_types			(GstPad *pad);
+const GstQueryType*	gst_pad_get_query_types_default		(GstPad *pad);
+gboolean		gst_pad_query				(GstPad *pad, GstQueryType type,
 								 GstFormat *format, gint64 *value);
-gboolean 		gst_pad_query_default 			(GstPad *pad, GstPadQueryType type,
+gboolean 		gst_pad_query_default 			(GstPad *pad, GstQueryType type,
 		                      				 GstFormat *format, gint64 *value);
 
 void			gst_pad_set_internal_connection_function(GstPad *pad, GstPadIntConnFunction intconn);
@@ -474,6 +483,10 @@ GList*	 		gst_pad_get_internal_connections_default (GstPad *pad);
 gboolean 		gst_pad_dispatcher 			(GstPad *pad, GstPadDispatcherFunction dispatch, 
 								 gpointer data);
 
+#define			gst_pad_add_probe(pad, probe) \
+			(gst_probe_dispatcher_add_probe (&(GST_REAL_PAD (pad)-probedisp), probe))
+#define			gst_pad_remove_probe(pad, probe) \
+			(gst_probe_dispatcher_remove_probe (&(GST_REAL_PAD (pad)-probedisp), probe))
 
 #ifndef GST_DISABLE_LOADSAVE
 void			gst_pad_load_and_connect		(xmlNodePtr self, GstObject *parent);
