@@ -23,8 +23,12 @@
 /*#define GST_DEBUG_ENABLED */
 #include <gst/gst.h>
 
-#include "cothreads_compat.h"
-
+#ifdef USE_COTHREADS
+# include "cothreads_compat.h"
+#else
+# define COTHREADS_NAME_CAPITAL ""
+# define COTHREADS_NAME 	""
+#endif
 
 #define GST_ELEMENT_SCHED_CONTEXT(elem)		((GstOptSchedulerCtx*) (GST_ELEMENT_CAST (elem)->sched_private))
 #define GST_ELEMENT_SCHED_GROUP(elem)		(GST_ELEMENT_SCHED_CONTEXT (elem)->group)
@@ -63,8 +67,9 @@ struct _GstOptScheduler {
 
   GstOptSchedulerState	 state;
 
+#ifdef USE_COTHREADS
   cothread_context 	*context;
-  gboolean		 use_cothreads;
+#endif
   gint 			 iterations;
 
   GSList 		*elements;
@@ -72,6 +77,8 @@ struct _GstOptScheduler {
 
   GList			*runqueue;
   gint			 recursion;
+
+  gint			 max_recursion;
 };
 
 struct _GstOptSchedulerClass {
@@ -118,6 +125,10 @@ typedef enum {
   GST_OPT_SCHEDULER_GROUP_LOOP			= 2,
 } GstOptSchedulerGroupType;
 
+#define GST_OPT_SCHEDULER_GROUP_SET_FLAG(group,flag) 	((group)->flags |= (flag))
+#define GST_OPT_SCHEDULER_GROUP_UNSET_FLAG(group,flag) 	((group)->flags &= (flag))
+#define GST_OPT_SCHEDULER_GROUP_IS_FLAG_SET(group,flag) ((group)->flags & (flag))
+
 #define GST_OPT_SCHEDULER_GROUP_DISABLE(group) 		((group)->flags |= GST_OPT_SCHEDULER_GROUP_DISABLED)
 #define GST_OPT_SCHEDULER_GROUP_ENABLE(group) 		((group)->flags &= ~GST_OPT_SCHEDULER_GROUP_DISABLED)
 #define GST_OPT_SCHEDULER_GROUP_IS_ENABLED(group) 	(!((group)->flags & GST_OPT_SCHEDULER_GROUP_DISABLED))
@@ -140,7 +151,9 @@ struct _GstOptSchedulerGroup {
   GSList			*providers;		/* other groups that provide data
 							   for this group */
 
+#ifdef USE_COTHREADS
   cothread 			*cothread;		/* the cothread of this group */
+#endif
   GroupScheduleFunction 	 schedulefunc;
   int				 argc;
   char			       **argv;
@@ -159,14 +172,13 @@ struct _GstOptSchedulerCtx {
   GstOptSchedulerGroup *group;  			/* the group this element belongs to */
 
   GstOptSchedulerCtxFlags flags;			/* flags for this element */
-  gint element_type;
 };
 
 enum
 {
   ARG_0,
-  ARG_USE_COTHREADS,
   ARG_ITERATIONS,
+  ARG_MAX_RECURSION,
 };
  
 
@@ -186,6 +198,7 @@ static void		gst_opt_scheduler_add_element		(GstScheduler *sched, GstElement *el
 static void     	gst_opt_scheduler_remove_element	(GstScheduler *sched, GstElement *element);
 static GstElementStateReturn  
 			gst_opt_scheduler_state_transition	(GstScheduler *sched, GstElement *element, gint transition);
+static void             gst_opt_scheduler_scheduling_change     (GstScheduler *sched, GstElement *element);
 static void 		gst_opt_scheduler_lock_element 		(GstScheduler *sched, GstElement *element);
 static void 		gst_opt_scheduler_unlock_element 	(GstScheduler *sched, GstElement *element);
 static void 		gst_opt_scheduler_yield 		(GstScheduler *sched, GstElement *element);
@@ -195,7 +208,7 @@ static void     	gst_opt_scheduler_pad_connect		(GstScheduler *sched, GstPad *sr
 static void     	gst_opt_scheduler_pad_disconnect 	(GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad);
 static GstPad*  	gst_opt_scheduler_pad_select 		(GstScheduler *sched, GList *padlist);
 static GstClockReturn   gst_opt_scheduler_clock_wait        	(GstScheduler *sched, GstElement *element,
-	                                                         GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter);
+	                                                         GstClockID id, GstClockTimeDiff *jitter);
 static GstSchedulerState
 			gst_opt_scheduler_iterate    		(GstScheduler *sched);
 
@@ -243,18 +256,23 @@ gst_opt_scheduler_class_init (GstOptSchedulerClass *klass)
   gobject_class->get_property   = GST_DEBUG_FUNCPTR (gst_opt_scheduler_get_property);
   gobject_class->dispose	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_dispose);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USE_COTHREADS,
-    g_param_spec_boolean ("use_cothreads", "Use cothreads", "Should this scheduler use cothreads",
-                          TRUE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USE_COTHREADS,
-    g_param_spec_int ("iterations", "Iterations", "Number of groups to schedule in one iteration (-1 == until EOS/error)",
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ITERATIONS,
+    g_param_spec_int ("iterations", "Iterations", 
+	    	      "Number of groups to schedule in one iteration (-1 == until EOS/error)",
                       -1, G_MAXINT, 1, G_PARAM_READWRITE));
+#ifndef USE_COTHREADS
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_RECURSION,
+    g_param_spec_int ("max_recursion", "Max recursion", 
+	    	      "Maximum number of recursions",
+                      1, G_MAXINT, 100, G_PARAM_READWRITE));
+#endif
 
   gstscheduler_class->setup             = GST_DEBUG_FUNCPTR (gst_opt_scheduler_setup);
   gstscheduler_class->reset             = GST_DEBUG_FUNCPTR (gst_opt_scheduler_reset);
   gstscheduler_class->add_element 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_add_element);
   gstscheduler_class->remove_element 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_remove_element);
   gstscheduler_class->state_transition 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_state_transition);
+  gstscheduler_class->scheduling_change = GST_DEBUG_FUNCPTR (gst_opt_scheduler_scheduling_change);
   gstscheduler_class->lock_element 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_lock_element);
   gstscheduler_class->unlock_element 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_unlock_element);
   gstscheduler_class->yield	 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_yield);
@@ -272,9 +290,8 @@ static void
 gst_opt_scheduler_init (GstOptScheduler *scheduler)
 {
   scheduler->elements = NULL;
-  //scheduler->use_cothreads = FALSE;
-  scheduler->use_cothreads = TRUE;
   scheduler->iterations = 1;
+  scheduler->max_recursion = 100;
 }
 
 static void
@@ -312,13 +329,14 @@ GstPluginDesc plugin_desc = {
 
 
 static void
-delete_chain (GstOptScheduler *osched, GstOptSchedulerChain *chain)
+delete_chain (GstOptSchedulerChain *chain)
 {
   GSList *groups;
+  GstOptScheduler *osched;
   
   GST_INFO (GST_CAT_SCHEDULING, "delete chain %p", chain);
 
-  g_assert (chain->sched == osched);
+  osched = chain->sched;
 
   osched->chains = g_slist_remove (osched->chains, chain);
 
@@ -326,7 +344,9 @@ delete_chain (GstOptScheduler *osched, GstOptSchedulerChain *chain)
   while (groups) {
     GstOptSchedulerGroup *group = (GstOptSchedulerGroup *) groups->data;
 
-    group->chain = NULL;
+    /* clear all group's chain pointers, so they can never reference us again */
+    if (group->chain == chain)
+      group->chain = NULL;
     
     groups = g_slist_next (groups);
   }
@@ -335,7 +355,7 @@ delete_chain (GstOptScheduler *osched, GstOptSchedulerChain *chain)
   g_free (chain);
 }
 
-static void
+static GstOptSchedulerChain*
 add_to_chain (GstOptSchedulerChain *chain, GstOptSchedulerGroup *group)
 {
   GST_INFO (GST_CAT_SCHEDULING, "adding group %p to chain %p", group, chain);
@@ -345,6 +365,8 @@ add_to_chain (GstOptSchedulerChain *chain, GstOptSchedulerGroup *group)
   chain->groups = g_slist_prepend (chain->groups, group);
   group->chain = chain;
   chain->num_groups++;
+
+  return chain;
 }
 
 static GstOptSchedulerChain*
@@ -362,56 +384,90 @@ create_chain (GstOptScheduler *osched)
   return chain;
 }
 
-static void
+static GstOptSchedulerChain*
 remove_from_chain (GstOptSchedulerChain *chain, GstOptSchedulerGroup *group)
 {
   GST_INFO (GST_CAT_SCHEDULING, "removing group %p from chain %p", group, chain);
 
+  if (!chain)
+    return NULL;
+
+  g_assert (group);
   g_assert (group->chain == chain);
 
   chain->groups = g_slist_remove (chain->groups, group);
   chain->num_groups--;
 
   group->chain = NULL;
+
+  if (chain->num_groups == 0) {
+    GST_INFO (GST_CAT_SCHEDULING, "chain %p is empty, removing", chain);
+    delete_chain (chain);
+
+    return NULL;
+  }
+
+  return chain;
 }
 
-static void
+static GstOptSchedulerChain*
 merge_chains (GstOptSchedulerChain *chain1, GstOptSchedulerChain *chain2)
 {
   GSList *walk;
+
+  g_assert (chain1 != NULL);
   
-  GST_INFO (GST_CAT_SCHEDULING, "mergin chain %p and %p", chain1, chain2);
+  GST_INFO (GST_CAT_SCHEDULING, "merging chain %p and %p", chain1, chain2);
   
-  if (chain1 == chain2)
-    return;
+  if (chain1 == chain2 || chain2 == NULL)
+    return chain1;
 
   walk = chain2->groups;
   while (walk) {
     GstOptSchedulerGroup *group = (GstOptSchedulerGroup *) walk->data;
+    walk = g_slist_next (walk);
 
     group->chain = NULL;
     add_to_chain (chain1, group);
-    walk = g_slist_next (walk);
   }
-  delete_chain (chain2->sched, chain2);
+  delete_chain (chain2);
+
+  return chain1;
 }
 
 static void
 chain_group_set_enabled (GstOptSchedulerChain *chain, GstOptSchedulerGroup *group, gboolean enabled)
 {
+  g_assert (chain != NULL);
+  g_assert (group != NULL);
+
+  GST_INFO (GST_CAT_SCHEDULING, "request to %d group %p in chain %p, have %d groups enabled out of %d", 
+		  enabled, group, chain, chain->num_enabled, chain->num_groups);
+
+  if (enabled)
+    GST_OPT_SCHEDULER_GROUP_ENABLE (group);
+  else 
+    GST_OPT_SCHEDULER_GROUP_DISABLE (group);
+
   if (enabled) {
-    chain->num_enabled++;
+    if (chain->num_enabled < chain->num_groups)
+      chain->num_enabled++;
+
     GST_INFO (GST_CAT_SCHEDULING, "enable group %p in chain %p, now %d groups enabled out of %d", group, chain,
 		    chain->num_enabled, chain->num_groups);
+
     if (chain->num_enabled == chain->num_groups) {
       GST_INFO (GST_CAT_SCHEDULING, "enable chain %p", chain);
       GST_OPT_SCHEDULER_CHAIN_ENABLE (chain);
     }
   }
   else {
-    chain->num_enabled--;
+    if (chain->num_enabled > 0)
+      chain->num_enabled--;
+
     GST_INFO (GST_CAT_SCHEDULING, "disable group %p in chain %p, now %d groups enabled out of %d", group, chain,
 		    chain->num_enabled, chain->num_groups);
+
     if (chain->num_enabled == 0) {
       GST_INFO (GST_CAT_SCHEDULING, "disable chain %p", chain);
       GST_OPT_SCHEDULER_CHAIN_DISABLE (chain);
@@ -419,14 +475,17 @@ chain_group_set_enabled (GstOptSchedulerChain *chain, GstOptSchedulerGroup *grou
   }
 }
 
-static void
+static GstOptSchedulerGroup*
 add_to_group (GstOptSchedulerGroup *group, GstElement *element)
 {
+  g_assert (group != NULL);
+  g_assert (element != NULL);
+
   GST_INFO (GST_CAT_SCHEDULING, "adding element \"%s\" to group %p", GST_ELEMENT_NAME (element), group);
 
   if (GST_ELEMENT_IS_DECOUPLED (element)) {
     GST_INFO (GST_CAT_SCHEDULING, "element \"%s\" is decoupled, not adding to group %p", GST_ELEMENT_NAME (element), group);
-    return;
+    return group;
   }
 
   g_assert (GST_ELEMENT_SCHED_GROUP (element) == NULL);
@@ -435,6 +494,8 @@ add_to_group (GstOptSchedulerGroup *group, GstElement *element)
   group->num_elements++;
 
   GST_ELEMENT_SCHED_GROUP (element) = group;
+
+  return group;
 }
 
 static GstOptSchedulerGroup*
@@ -454,17 +515,21 @@ create_group (GstOptSchedulerChain *chain, GstElement *element)
 static void 
 destroy_group_scheduler (GstOptSchedulerGroup *group) 
 {
+  g_assert (group);
+
   if (group->flags & GST_OPT_SCHEDULER_GROUP_RUNNING)
     g_warning ("removing running element");
 
+#ifdef USE_COTHREADS
   if (group->cothread) {
     do_cothread_destroy (group->cothread);
+    group->cothread = NULL;
   }
-  else {
-    group->schedulefunc = NULL;
-    group->argc = 0;
-    group->argv = NULL;
-  }
+#else
+  group->schedulefunc = NULL;
+  group->argc = 0;
+  group->argv = NULL;
+#endif
 
   group->flags &= ~GST_OPT_SCHEDULER_GROUP_SCHEDULABLE;
 }
@@ -476,6 +541,7 @@ delete_group (GstOptSchedulerGroup *group)
   
   GST_INFO (GST_CAT_SCHEDULING, "delete group %p", group);
 
+  g_assert (group != NULL);
   g_assert (group->chain == NULL);
 
   if (group->flags & GST_OPT_SCHEDULER_GROUP_SCHEDULABLE)
@@ -495,45 +561,54 @@ delete_group (GstOptSchedulerGroup *group)
   g_free (group);
 }
 
-static void
-merge_groups (GstOptSchedulerGroup *group1, GstOptSchedulerGroup *group2)
-{
-  GSList *walk;
-  GstOptSchedulerChain *chain1;
-  GstOptSchedulerChain *chain2;
-  
-  GST_INFO (GST_CAT_SCHEDULING, "merging groups %p and %p", group1, group2);
-  
-  if (group1 == group2)
-    return;
-
-  walk = group2->elements;
-  while (walk) {
-    add_to_group (group1, (GstElement *)walk->data);
-    walk = g_slist_next (walk);
-  }
-
-  chain1 = group1->chain;
-  chain2 = group2->chain;
-
-  remove_from_chain (chain2, group2);
-  delete_group (group2);
-
-  merge_chains (chain1, chain2);
-}
-
-/*
-static void
+static GstOptSchedulerGroup*
 remove_from_group (GstOptSchedulerGroup *group, GstElement *element)
 {
-  GST_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" to group %p", GST_ELEMENT_NAME (element), group);
+  GST_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" from group %p", GST_ELEMENT_NAME (element), group);
+
+  g_assert (group != NULL);
+  g_assert (element != NULL);
 
   group->elements = g_slist_remove (group->elements, element);
   group->num_elements--;
 
   GST_ELEMENT_SCHED_GROUP (element) = NULL;
+
+  if (group->num_elements == 0) {
+    GST_INFO (GST_CAT_SCHEDULING, "group %p is empty, deleting", group);
+    remove_from_chain (group->chain, group);
+    delete_group (group);
+    return NULL;
+  }
+  return group;
+}
+
+static GstOptSchedulerGroup*
+merge_groups (GstOptSchedulerGroup *group1, GstOptSchedulerGroup *group2)
+{
+  g_assert (group1 != NULL);
+
+  GST_INFO (GST_CAT_SCHEDULING, "merging groups %p and %p", group1, group2);
+  
+  if (group1 == group2 || group2 == NULL)
+    return group1;
+
+  while (group2) {
+    GstElement *element = (GstElement *)group2->elements->data;
+   
+    group2 = remove_from_group (group2, element);
+    add_to_group (group1, element);
+  }
+  
+  return group1;
+}
+
+static void
+group_error_handler (GstOptSchedulerGroup *group) 
+{
+  chain_group_set_enabled (group->chain, group, FALSE);
+  group->chain->sched->state = GST_OPT_SCHEDULER_STATE_ERROR;
 }
-*/
 
 /* this function enables/disables an element, it will set/clear a flag on the element 
  * and tells the chain that the group is enabled if all elements inside the group are
@@ -541,23 +616,33 @@ remove_from_group (GstOptSchedulerGroup *group, GstElement *element)
 static void
 group_element_set_enabled (GstOptSchedulerGroup *group, GstElement *element, gboolean enabled)
 {
+  g_assert (group != NULL);
+  g_assert (element != NULL);
+
+  GST_INFO (GST_CAT_SCHEDULING, "request to %d element %s in group %p, have %d elements enabled out of %d", 
+		    enabled, GST_ELEMENT_NAME (element), group, group->num_enabled, group->num_elements);
+
   if (enabled) {
-    group->num_enabled++;
+    if (group->num_enabled < group->num_elements)
+      group->num_enabled++;
+
     GST_INFO (GST_CAT_SCHEDULING, "enable element %s in group %p, now %d elements enabled out of %d", 
 		    GST_ELEMENT_NAME (element), group, group->num_enabled, group->num_elements);
+
     if (group->num_enabled == group->num_elements) {
       GST_INFO (GST_CAT_SCHEDULING, "enable group %p", group);
-      GST_OPT_SCHEDULER_GROUP_ENABLE (group);
       chain_group_set_enabled (group->chain, group, TRUE);
     }
   }
   else {
-    group->num_enabled--;
+    if (group->num_enabled > 0)
+      group->num_enabled--;
+
     GST_INFO (GST_CAT_SCHEDULING, "disable element %s in group %p, now %d elements enabled out of %d", 
 		    GST_ELEMENT_NAME (element), group, group->num_enabled, group->num_elements);
+
     if (group->num_enabled == 0) {
       GST_INFO (GST_CAT_SCHEDULING, "disable group %p", group);
-      GST_OPT_SCHEDULER_GROUP_DISABLE (group);
       chain_group_set_enabled (group->chain, group, FALSE);
     }
   }
@@ -570,23 +655,33 @@ group_element_set_enabled (GstOptSchedulerGroup *group, GstElement *element, gbo
 static gboolean 
 schedule_group (GstOptSchedulerGroup *group) 
 {
-  if (group->chain->sched->use_cothreads) {
-    if (group->cothread)
-      do_cothread_switch (group->cothread);
-    return TRUE;
-  }
-  else {
-    group->schedulefunc (group->argc, group->argv);
-    return TRUE;
-  }
-  return FALSE;
+  if (!group->entry)
+    return FALSE;
+
+#ifdef USE_COTHREADS
+  if (group->cothread)
+    do_cothread_switch (group->cothread);
+  else
+    g_warning ("(internal error): trying to schedule group without cothread");
+  return TRUE;
+#else
+  group->schedulefunc (group->argc, group->argv);
+  return TRUE;
+#endif
 }
 
+#ifndef USE_COTHREADS
 static void
 gst_opt_scheduler_schedule_run_queue (GstOptScheduler *osched)
 {
   GST_INFO (GST_CAT_SCHEDULING, "entering scheduler run queue recursion %d", osched->recursion);
 
+  /* make sure we don't exceed max_recursion */
+  if (osched->recursion > osched->max_recursion) {
+    osched->state = GST_OPT_SCHEDULER_STATE_ERROR;
+    return;
+  }
+
   osched->recursion++;
 
   while (osched->runqueue) {
@@ -597,7 +692,10 @@ gst_opt_scheduler_schedule_run_queue (GstOptScheduler *osched)
 
     GST_INFO (GST_CAT_SCHEDULING, "scheduling %p", group);
 
-    schedule_group (group);
+    if (!schedule_group (group)) {
+      g_warning  ("error scheduling group %p", group);
+      group_error_handler (group);
+    }
 
     GST_INFO (GST_CAT_SCHEDULING, "done scheduling %p", group);
   }
@@ -606,12 +704,17 @@ gst_opt_scheduler_schedule_run_queue (GstOptScheduler *osched)
 
   osched->recursion--;
 }
+#endif
 
 /* a chain is scheduled by picking the first active group and scheduling it */
 static void 
 schedule_chain (GstOptSchedulerChain *chain) 
 {
-  GSList *groups = chain->groups;
+  GSList *groups;
+  GstOptScheduler *osched;
+
+  osched = chain->sched;
+  groups = chain->groups;
 
   while (groups) {
     GstOptSchedulerGroup *group = (GstOptSchedulerGroup *) groups->data;
@@ -619,21 +722,17 @@ schedule_chain (GstOptSchedulerChain *chain)
     groups = g_slist_next (groups);
 
     if (!GST_OPT_SCHEDULER_GROUP_IS_DISABLED (group)) {
-      GstOptScheduler *osched;
-
-      osched = chain->sched;
 
       GST_INFO (GST_CAT_SCHEDULING, "scheduling group %p in chain %p", 
  	        group, chain);
 
-      if (osched->use_cothreads) {
-	schedule_group (group);
-      }
-      else {
-        osched->recursion = 0;
-        osched->runqueue = g_list_append (osched->runqueue, group);
-        gst_opt_scheduler_schedule_run_queue (osched);
-      }
+#ifdef USE_COTHREADS
+      schedule_group (group);
+#else
+      osched->recursion = 0;
+      osched->runqueue = g_list_append (osched->runqueue, group);
+      gst_opt_scheduler_schedule_run_queue (osched);
+#endif
 
       GST_INFO (GST_CAT_SCHEDULING, "done scheduling group %p in chain %p", 
  	        group, chain);
@@ -710,8 +809,7 @@ unkown_group_schedule_function (int argc, char *argv[])
   GstOptSchedulerGroup *group = (GstOptSchedulerGroup *) argv;
 
   g_warning ("(internal error) unkown group type %d, disabling\n", group->type);
-  chain_group_set_enabled (group->chain, group, FALSE);
-  group->chain->sched->state = GST_OPT_SCHEDULER_STATE_ERROR;
+  group_error_handler (group);
 
   return 0;
 }
@@ -731,21 +829,21 @@ gst_opt_scheduler_loop_wrapper (GstPad *sinkpad, GstBuffer *buffer)
   osched = group->chain->sched;
 
 
-  if (osched->use_cothreads) {
-    if (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad))) {
-      g_warning ("deadlock detected, disabling group %p", group);
-      chain_group_set_enabled (group->chain, group, FALSE);
-      group->chain->sched->state = GST_OPT_SCHEDULER_STATE_ERROR;
-    }
-    else {
-      GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)) = g_list_append (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)), buffer);
-      schedule_group (group);
-    }
+#ifdef USE_COTHREADS
+  if (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad))) {
+    g_warning ("deadlock detected, disabling group %p", group);
+    group_error_handler (group);
   }
-  else if (!(group->flags & GST_OPT_SCHEDULER_GROUP_RUNNING)) {
+  else {
     GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)) = g_list_append (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)), buffer);
+    schedule_group (group);
+  }
+#else
+  GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)) = g_list_append (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad)), buffer);
+  if (!(group->flags & GST_OPT_SCHEDULER_GROUP_RUNNING)) {
     osched->runqueue = g_list_append (osched->runqueue, group);
   }
+#endif
   
   GST_INFO (GST_CAT_SCHEDULING, "after loop wrapper buflist %d", 
 	    g_list_length (GST_PAD_BUFLIST (GST_RPAD_PEER (sinkpad))));
@@ -771,19 +869,19 @@ gst_opt_scheduler_get_wrapper (GstPad *srcpad)
     group = GST_ELEMENT_SCHED_GROUP (GST_PAD_PARENT (srcpad));
     osched = group->chain->sched;
 
-    if (osched->use_cothreads) {
-      schedule_group (group);
-    }
-    else if (!(group->flags & GST_OPT_SCHEDULER_GROUP_RUNNING)) {
+#ifdef USE_COTHREADS
+    schedule_group (group);
+#else
+    if (!(group->flags & GST_OPT_SCHEDULER_GROUP_RUNNING)) {
       osched->runqueue = g_list_append (osched->runqueue, group);
       gst_opt_scheduler_schedule_run_queue (osched);
     }
     else {
       g_warning ("deadlock detected, disabling group %p", group);
-      chain_group_set_enabled (group->chain, group, FALSE);
-      group->chain->sched->state = GST_OPT_SCHEDULER_STATE_ERROR;
+      group_error_handler (group);
       return NULL;
     }
+#endif
     
     if (GST_PAD_BUFLIST (srcpad)) {
       buffer = (GstBuffer *) GST_PAD_BUFLIST (srcpad)->data;
@@ -826,21 +924,20 @@ setup_group_scheduler (GstOptScheduler *osched, GstOptSchedulerGroup *group)
   else if (group->type == GST_OPT_SCHEDULER_GROUP_LOOP)
     wrapper = loop_group_schedule_function;
 	
-  if (osched->use_cothreads) {
-    if (!(group->flags & GST_OPT_SCHEDULER_GROUP_SCHEDULABLE)) {
-      do_cothread_create (group->cothread, osched->context,
+#ifdef USE_COTHREADS
+  if (!(group->flags & GST_OPT_SCHEDULER_GROUP_SCHEDULABLE)) {
+    do_cothread_create (group->cothread, osched->context,
  		      (cothread_func) wrapper, 0, (char **) group);
-    }
-    else {
-      do_cothread_setfunc (group->cothread, osched->context,
- 		      (cothread_func) wrapper, 0, (char **) group);
-    }
   }
   else {
-    group->schedulefunc = wrapper;
-    group->argc = 0;
-    group->argv = (char **) group;
+    do_cothread_setfunc (group->cothread, osched->context,
+ 		      (cothread_func) wrapper, 0, (char **) group);
   }
+#else
+  group->schedulefunc = wrapper;
+  group->argc = 0;
+  group->argv = (char **) group;
+#endif
   group->flags |= GST_OPT_SCHEDULER_GROUP_SCHEDULABLE;
 }
 
@@ -905,6 +1002,12 @@ gst_opt_scheduler_state_transition (GstScheduler *sched, GstElement *element, gi
   return res;
 }
 
+static void
+gst_opt_scheduler_scheduling_change (GstScheduler *sched, GstElement *element)
+{
+  g_warning ("scheduling change, implement me");
+}
+
 static void
 get_group (GstElement *element, GstOptSchedulerGroup **group)
 {
@@ -988,24 +1091,42 @@ typedef enum {
 static void
 gst_opt_scheduler_setup (GstScheduler *sched)
 {   
+#ifdef USE_COTHREADS
   GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
 	      
   /* first create thread context */
-  if (osched->context == NULL && osched->use_cothreads) {
+  if (osched->context == NULL) {
     GST_DEBUG (GST_CAT_SCHEDULING, "initializing cothread context");
     osched->context = do_cothread_context_init ();
   }
+#endif
 } 
   
 static void 
 gst_opt_scheduler_reset (GstScheduler *sched)
 { 
+#ifdef USE_COTHREADS
   GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GSList *chains = osched->chains;
+
+  while (chains) {
+    GstOptSchedulerChain *chain = (GstOptSchedulerChain *) chains->data;
+    GSList *groups = chain->groups;
+
+    while (groups) {
+      GstOptSchedulerGroup *group = (GstOptSchedulerGroup *) groups->data;
+
+      destroy_group_scheduler (group);
+      groups = groups->next;
+    }
+    chains = chains->next;
+  }
 	      
-  if (osched->context && osched->use_cothreads) {
+  if (osched->context) {
     do_cothread_context_destroy (osched->context);
     osched->context = NULL; 
   }
+#endif
 }     
 static void
 gst_opt_scheduler_add_element (GstScheduler *sched, GstElement *element)
@@ -1041,12 +1162,11 @@ gst_opt_scheduler_add_element (GstScheduler *sched, GstElement *element)
 static void
 gst_opt_scheduler_remove_element (GstScheduler *sched, GstElement *element)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
   GstOptSchedulerGroup *group;
 
   GST_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" from scheduler", GST_ELEMENT_NAME (element));
 
-  /* decoupled elements are not added to the scheduler lists and should therefor
+  /* decoupled elements are not added to the scheduler lists and should therefore
    * no be removed */
   if (GST_ELEMENT_IS_DECOUPLED (element))
     return;
@@ -1054,18 +1174,14 @@ gst_opt_scheduler_remove_element (GstScheduler *sched, GstElement *element)
   /* the element is guaranteed to live in it's own group/chain now */
   get_group (element, &group);
   if (group) {
+    GstOptSchedulerChain *chain;
     
     GST_ELEMENT_SCHED_GROUP (element) = NULL;
 
-    if (group->chain) {
-      GstOptSchedulerChain *chain;
-
-      chain = group->chain;
-
+    chain = group->chain;
+    if (chain) {
       remove_from_chain (chain, group);
-      delete_chain (osched, chain);
     }
-
     delete_group (group);
   }
 
@@ -1090,23 +1206,35 @@ gst_opt_scheduler_unlock_element (GstScheduler *sched, GstElement *element)
 static void
 gst_opt_scheduler_yield (GstScheduler *sched, GstElement *element)
 {
-  //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+#ifdef USE_COTHREADS
+  /* yield hands control to the main cothread context if the requesting 
+   * element is the entry point of the group */
+  GstOptSchedulerGroup *group;
+  get_group (element, &group);
+  if (group && group->entry == element)
+    do_cothread_switch (do_cothread_get_main (((GstOptScheduler*)sched)->context)); 
+#endif
 }
 
 static gboolean
 gst_opt_scheduler_interrupt (GstScheduler *sched, GstElement *element)
 {
-  //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
-  
-  g_warning ("interrupt element, implement me");
-
+#ifdef USE_COTHREADS
+  do_cothread_switch (do_cothread_get_main (((GstOptScheduler*)sched)->context)); 
+  return FALSE;
+#else
   return TRUE;
+#endif
 }
 
 static void
 gst_opt_scheduler_error (GstScheduler *sched, GstElement *element)
 {
   GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptSchedulerGroup *group;
+  get_group (element, &group);
+  if (group)
+    group_error_handler (group);
 
   osched->state = GST_OPT_SCHEDULER_STATE_ERROR;
 }
@@ -1125,7 +1253,7 @@ gst_opt_scheduler_pad_connect (GstScheduler *sched, GstPad *srcpad, GstPad *sink
   element1 = GST_PAD_PARENT (srcpad);
   element2 = GST_PAD_PARENT (sinkpad);
 
-  /* first we need to figure out whar type of connection we're dealing
+  /* first we need to figure out what type of connection we're dealing
    * with */
   if (element1->loopfunc && element2->loopfunc)
     type = GST_OPT_LOOP_TO_LOOP;
@@ -1140,11 +1268,17 @@ gst_opt_scheduler_pad_connect (GstScheduler *sched, GstPad *srcpad, GstPad *sink
 	/* this could be tricky, the get based source could 
 	 * already be part of a loop based group in another pad,
 	 * we assert on that for now */
-	if (GST_ELEMENT_SCHED_CONTEXT (element1) &&
+	if (GST_ELEMENT_SCHED_CONTEXT (element1) != NULL &&
 	    GST_ELEMENT_SCHED_GROUP (element1) != NULL) 
 	{
-          g_warning ("internal error: cannot schedule get to loop with get in group");
-	  return;
+          GstOptSchedulerGroup *group = GST_ELEMENT_SCHED_GROUP (element1);
+
+	  /* if the loop based element is the entry point we're ok, if it
+	   * isn't then we have multiple loop based elements in this group */
+	  if (group->entry != element2) {
+            g_error ("internal error: cannot schedule get to loop in multi-loop based group");
+	    return;
+	  }
 	}
       }
       else
@@ -1154,13 +1288,18 @@ gst_opt_scheduler_pad_connect (GstScheduler *sched, GstPad *srcpad, GstPad *sink
       if (GST_RPAD_GETFUNC (srcpad) && GST_RPAD_CHAINFUNC (sinkpad)) {
         type = GST_OPT_GET_TO_CHAIN;
 	/* the get based source could already be part of a loop 
-	 * based group in another pad,
-	 * we assert on that for now */
-	if (GST_ELEMENT_SCHED_CONTEXT (element1) &&
+	 * based group in another pad, we assert on that for now */
+	if (GST_ELEMENT_SCHED_CONTEXT (element1) != NULL &&
 	    GST_ELEMENT_SCHED_GROUP (element1) != NULL) 
 	{
-          g_warning ("internal error: cannot schedule get to loop with get in group");
-	  return;
+          GstOptSchedulerGroup *group = GST_ELEMENT_SCHED_GROUP (element1);
+
+	  /* if the get based element is the entry point we're ok, if it
+	   * isn't then we have a mixed loop/chain based group */
+	  if (group->entry != element1) {
+            g_error ("internal error: cannot schedule get to chain with mixed loop/chain based group");
+	    return;
+	  }
 	}
       }
       else 
@@ -1236,6 +1375,9 @@ gst_opt_scheduler_pad_connect (GstScheduler *sched, GstPad *srcpad, GstPad *sink
 
       group1 = GST_ELEMENT_SCHED_GROUP (element1);
       group2 = GST_ELEMENT_SCHED_GROUP (element2);
+
+      g_assert (group2 != NULL);
+
       /* group2 is guaranteed to exist as it contains a loop-based element.
        * group1 only exists if element1 is connected to some other element */
       if (!group1) {
@@ -1252,13 +1394,17 @@ gst_opt_scheduler_pad_connect (GstScheduler *sched, GstPad *srcpad, GstPad *sink
       break;
     }
     case GST_OPT_INVALID:
-      g_warning ("(internal error) invalid element connection");
+      g_error ("(internal error) invalid element connection, what are you doing?");
       break;
   }
 }
 
+/* 
+ * checks if an element is still connected to some other element in the group. 
+ * no checking is done on the brokenpad arg 
+ * */
 static gboolean
-element_has_connection_with_group (GstElement *element, GstOptSchedulerGroup *group)
+element_has_connection_with_group (GstElement *element, GstOptSchedulerGroup *group, GstPad *brokenpad)
 {
   gboolean connected = FALSE;
   const GList *pads;
@@ -1269,11 +1415,29 @@ element_has_connection_with_group (GstElement *element, GstOptSchedulerGroup *gr
     GstPad *pad = GST_PAD_CAST (pads->data);
     pads = g_list_next (pads);
 
-    /* we only operate on real pads */
-    if (!GST_IS_REAL_PAD (pad))
+    /* we only operate on real pads and on the pad that is not broken */
+    if (!GST_IS_REAL_PAD (pad) || pad == brokenpad)
       continue;
 
     if (GST_PAD_PEER (pad)) {
+      GstElement *parent;
+      GstOptSchedulerGroup *parentgroup;
+
+      /* see in what group this element is */
+      parent = GST_PAD_PARENT (GST_PAD_PEER (pad));
+
+      /* connections with decoupled elements are valid */
+      if (GST_ELEMENT_IS_DECOUPLED (parent)) {
+        connected = TRUE;
+      }
+      else {
+	/* for non-decoupled elements we need to check the group */
+        get_group (parent, &parentgroup);
+
+        /* if it's in the same group, we're still connected */
+        if (parentgroup == group)
+          connected = TRUE;
+      }
     } 
   }
   return connected;
@@ -1285,7 +1449,6 @@ gst_opt_scheduler_pad_disconnect (GstScheduler *sched, GstPad *srcpad, GstPad *s
   //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
   GstElement *element1, *element2;
   GstOptSchedulerGroup *group1, *group2;
-  gboolean still_connect;
 
   GST_INFO (GST_CAT_SCHEDULING, "pad disconnect between \"%s:%s\" and \"%s:%s\"", 
 		  GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (sinkpad));
@@ -1296,37 +1459,65 @@ gst_opt_scheduler_pad_disconnect (GstScheduler *sched, GstPad *srcpad, GstPad *s
   get_group (element1, &group1);
   get_group (element2, &group2);
 
-  /* having no groups is pretty bad, this means that two decoupled 
-   * elements were connected or something */
-  if (!group1 && !group2) {
-    g_warning ("internal error: cannot disconnect pads");
+  /* if one the elements has no group (anymore) we don't really care 
+   * about the connection */
+  if (!group1 || !group2) {
+    GST_INFO (GST_CAT_SCHEDULING, "one (or both) of the elements is not in a group, not interesting");
     return;
   }
 
-  /* see if the group has to be broken up */
-  if (group1)
-    still_connect = element_has_connection_with_group (element2, group1);
-  else
-    still_connect = element_has_connection_with_group (element1, group2);
+  /* easy part, groups are different */
+  if (group1 != group2) {
+    GST_INFO (GST_CAT_SCHEDULING, "elements are in different groups");
 
-  /* if there is still a connection, we don't need to break this group */
-  if (still_connect)
-    return;
-
-  /* if they are equal, they both are non zero */
-  if (group1 == group2) {
-    g_warning ("pad disconnect: implement me");
-  }
-  else if (group1) {
-    g_warning ("pad disconnect: implement me");
+    /* FIXME, need to eventually break the chain */
+    g_warning ("pad disconnect for different groups, implement me");
   }
+  /* hard part, groups are equal */
   else {
-    /* there was no group for element1, see if the element
-     * was an entry point for group2 */
-    if (group2) {
-      if (group2->entry == element1) {
-        group2->entry = NULL;
+    gboolean still_connect1, still_connect2;
+    GstOptSchedulerGroup *group;
+    
+    /* since group1 == group2, it doesn't matter which group we take */
+    group = group1;
+
+    GST_INFO (GST_CAT_SCHEDULING, "elements are in the same group %p", group);
+
+    /* check if the element is still connected to some other element in the group,
+     * we pass the pad that is broken up as an arg because a connection on that pad
+     * is not valid anymore */
+    still_connect1 = element_has_connection_with_group (element1, group, srcpad);
+    still_connect2 = element_has_connection_with_group (element2, group, sinkpad);
+
+    /* if there is still a connection, we don't need to break this group */
+    if (still_connect1 && still_connect2) {
+      GST_INFO (GST_CAT_SCHEDULING, "elements still have connections with other elements in the group");
+
+      /* FIXME, need to check for breaking up the group */
+      return;
+    }
+
+    /* now check which one of the elements we can remove from the group */
+    if (!still_connect1) {
+      GST_INFO (GST_CAT_SCHEDULING, "element1 is separated from the group");
+      /* see if the element was an entry point for the group */
+      if (group->entry == element1) {
+	/* we're going to remove the element so we need to clear it as the
+	 * entry point */
+        group->entry = NULL;
       }
+      remove_from_group (group, element1);
+    }
+    if (!still_connect2) {
+      GST_INFO (GST_CAT_SCHEDULING, "element2 is separated from the group");
+
+      /* see if the element was an entry point for the group */
+      if (group->entry == element2) {
+	/* we're going to remove the element so we need to clear it as the
+	 * entry point */
+        group->entry = NULL;
+      }
+      remove_from_group (group, element2);
     }
   }
 }
@@ -1343,12 +1534,8 @@ gst_opt_scheduler_pad_select (GstScheduler *sched, GList *padlist)
 
 static GstClockReturn
 gst_opt_scheduler_clock_wait (GstScheduler *sched, GstElement *element,
-	                      GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter)
+	                      GstClockID id, GstClockTimeDiff *jitter)
 {
-  GstClockID id;
-  
-  id = gst_clock_new_single_shot_id (clock, time);
-
   return gst_clock_id_wait (id, jitter);
 }
 
@@ -1409,6 +1596,9 @@ gst_opt_scheduler_show (GstScheduler *sched)
   GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
   GSList *chains;
 
+  g_print ("iterations:    %d\n", osched->iterations);
+  g_print ("max recursion: %d\n", osched->max_recursion);
+
   chains = osched->chains;
   while (chains) {
     GstOptSchedulerChain *chain = (GstOptSchedulerChain *) chains->data;
@@ -1448,12 +1638,12 @@ gst_opt_scheduler_get_property (GObject *object, guint prop_id,
   osched = GST_OPT_SCHEDULER_CAST (object);
 
   switch (prop_id) {
-    case ARG_USE_COTHREADS:
-      g_value_set_boolean (value, osched->use_cothreads);
-      break;
     case ARG_ITERATIONS:
       g_value_set_int (value, osched->iterations);
       break;
+    case ARG_MAX_RECURSION:
+      g_value_set_int (value, osched->max_recursion);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1471,12 +1661,12 @@ gst_opt_scheduler_set_property (GObject *object, guint prop_id,
   osched = GST_OPT_SCHEDULER_CAST (object);
 
   switch (prop_id) {
-    case ARG_USE_COTHREADS:
-      osched->use_cothreads = g_value_get_boolean (value);
-      break;
     case ARG_ITERATIONS:
       osched->iterations = g_value_get_int (value);
       break;
+    case ARG_MAX_RECURSION:
+      osched->max_recursion = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
