@@ -1,5 +1,7 @@
 #include <string.h>
 #include <stdlib.h>
+#include <signal.h>
+#include <sys/wait.h>
 #include <gst/gst.h>
 
 static guint64 iterations = 0;
@@ -104,11 +106,83 @@ xmllaunch_parse_cmdline (const gchar **argv)
     return l->data;
 }
 
+extern volatile gboolean glib_on_error_halt;
+static void fault_restore(void);
+
+static void 
+fault_handler (int signum, siginfo_t *si, void *misc)
+{
+  int spinning = TRUE;
+
+  fault_restore ();
+
+  if (si->si_signo == SIGSEGV) {
+    g_print ("Caught SIGSEGV accessing address %p\n", si->si_addr);
+  }
+  else if (si->si_signo == SIGQUIT){
+    g_print ("Caught SIGQUIT\n");
+  }
+  else {
+    g_print ("signo:  %d\n", si->si_signo);
+    g_print ("errno:  %d\n", si->si_errno);
+    g_print ("code:   %d\n", si->si_code);
+  }
+
+  glib_on_error_halt = FALSE;
+  g_on_error_stack_trace ("gst-launch");
+
+  wait (NULL);
+
+#if 1
+  /* FIXME how do we know if we were run by libtool? */
+  g_print ("Spinning.  Please run 'gdb gst-launch %d' to continue debugging, "
+  	   "Ctrl-C to quit, or Ctrl-\\ to dump core.\n",
+  	    getpid ());
+  while (spinning) usleep (1000000);
+#else
+  /* This spawns a gdb and attaches it to gst-launch. */
+  {
+    char str[40];
+    sprintf (str, "gdb -quiet gst-launch %d", getpid ());
+    system (str);
+  }
+
+  _exit(0);
+#endif
+
+}
+
+static void 
+fault_restore (void)
+{
+  struct sigaction action;
+
+  memset (&action, 0, sizeof (action));
+  action.sa_handler = SIG_DFL;
+
+  sigaction(SIGSEGV, &action, NULL);
+  sigaction(SIGQUIT, &action, NULL);
+}
+
+static void 
+fault_setup (void)
+{
+  struct sigaction action;
+
+  memset (&action, 0, sizeof (action));
+  action.sa_sigaction = fault_handler;
+  action.sa_flags = SA_SIGINFO;
+
+  sigaction (SIGSEGV, &action, NULL);
+  sigaction (SIGQUIT, &action, NULL);
+}
+
 int
 main(int argc, char *argv[])
 {
   /* options */
   gboolean silent = FALSE;
+  gboolean no_fault = FALSE;
   gchar *savefile = NULL;
   gchar *exclude_args = NULL;
   struct poptOption options[] = {
@@ -118,15 +192,21 @@ main(int argc, char *argv[])
      "do not output status information of TYPE", "TYPE1,TYPE2,..."},
     {"output",	'o',  POPT_ARG_STRING|POPT_ARGFLAG_STRIP, &savefile, 0,
      "save xml representation of pipeline to FILE and exit", "FILE"},
+    {"no_fault", 'f',  POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   &no_fault,   0,
+     "Do not install a fault handler", NULL},
     POPT_TABLEEND
   };
 
   GstElement *pipeline;
   gchar **argvn;
   GError *error = NULL;
+  gint res = 0;
 
   free (malloc (8)); /* -lefence */
 
+  if (!no_fault)
+    fault_setup();
+
   gst_init_with_popt_table (&argc, &argv, options);
   
   /* make a null-terminated version of argv */
@@ -165,9 +245,10 @@ main(int argc, char *argv[])
     gst_event_print_stats();
 
     fprintf(stderr,"RUNNING pipeline\n");
-    if (gst_element_set_state (pipeline, GST_STATE_PLAYING) != GST_STATE_SUCCESS) {
+    if (gst_element_set_state (pipeline, GST_STATE_PLAYING) == GST_STATE_FAILURE) {
       fprintf(stderr,"pipeline doesn't want to play\n");
-      exit (-1);
+      res = -1;
+      goto end;
     }
 
     s_clock = gst_bin_get_clock (GST_BIN (pipeline));
@@ -182,11 +263,14 @@ main(int argc, char *argv[])
     }
 
     gst_element_set_state (pipeline, GST_STATE_NULL);
-    gst_buffer_print_stats();
-    gst_event_print_stats();
-
   }
+
+end:
+  gst_buffer_print_stats();
+  gst_event_print_stats();
+
   gst_object_unref (GST_OBJECT (pipeline));
 
-  return 0;
+  return res;
 }
+
